# MySQL C# 教程

> 原文： [http://zetcode.com/db/mysqlcsharptutorial/](http://zetcode.com/db/mysqlcsharptutorial/)

## 关于本教程

这是 MySQL 数据库的 C# 教程。 它涵盖了使用 C# 进行 MySQL 编程的基础。 在本教程中，我们使用 _Connector/Net_ 驱动程序。 该驱动程序基于 ADO.NET 规范。 这些示例是在 Ubuntu Linux 上创建和测试的。 在 ZetCode 上有一个类似的 [MySQL Visual Basic 教程](/db/mysqlvb/)。

如果您需要重新学习 C# 语言的知识，请在 ZetCode 上找到完整的 [C# 教程](/lang/csharp/)。

## 关于 MySQL 数据库

MySQL 是领先的开源数据库管理系统。 它是一个多用户，多线程的数据库管理系统。 MySQL 在网络上特别流行。 它是由 Linux，Apache，MySQL 和 PHP 组成的非常流行的 _LAMP_ 平台的一部分。 目前，MySQL 由 Oracle 拥有。 MySQL 数据库在最重要的 OS 平台上可用。 它可以在 BSD Unix，Linux，Windows 或 Mac OS 上运行。 维基百科和 YouTube 使用 MySQL。 这些站点每天管理数百万个查询。 MySQL 有两个版本：MySQL 服务器系统和 MySQL 嵌入式系统。

## 开始之前

在 Linux 上，我们需要安装几个包来执行本教程中的示例：`libmysql6.1-cil`，`mysql-server`和`mysql-client`。 我们还需要从 Mono 项目（从包或从源代码）安装 C# 编译器。

`libmysql6.1-cil`是 CLI 的 MySQL 数据库连接器。 它用 C# 编写，并且可用于所有 CLI 语言：C# ，Visual Basic，Boo 等。

```cs
$ ls /usr/lib/cli/MySql.Data-6.1/MySql.Data.dll 
/usr/lib/cli/MySql.Data-6.1/MySql.Data.dll

```

从技术角度来看，我们需要一个 DLL。 在 Ubuntu Linux 上，它位于上述路径下。 我们需要知道 DLL 库的路径。 汇编我们的例子。

如果您尚未安装 MySQL，则必须安装它。

```cs
$ sudo apt-get install mysql-server

```

此命令将安装 MySQL 服务器和其他各种包。 在安装包时，提示我们输入 MySQL 根帐户的密码。 要从源代码安装 MySQL，请查看 [MySQL 安装](http://zetcode.com/databases/mysqltutorial/installation/)页面。

```cs
$ service mysql status
mysql start/running, process 1238

```

我们检查 MySQL 服务器是否正在运行。 如果没有，我们需要启动服务器。

```cs
$ sudo -b /usr/local/mysql/bin/mysqld_safe

```

上面的命令使用 MySQL 服务器启动脚本启动 MySQL 服务器。 我们启动 MySQL 服务器的方式可能有所不同。 这取决于我们是否从源代码或包安装了 MySQL，也取决于 Linux 发行版。 有关更多信息，请查阅 [MySQL 的第一步](http://zetcode.com/databases/mysqltutorial/firststeps/)或您的 Linux 发行版信息。

接下来，我们将创建一个新的数据库用户和一个新的数据库。 我们使用`mysql`客户端。

```cs
$ mysql -u root -p
Enter password: 
Welcome to the MySQL monitor.  Commands end with ; or \g.
Your MySQL connection id is 30
Server version: 5.0.67-0ubuntu6 (Ubuntu)

Type 'help;' or '\h' for help. Type '\c' to clear the buffer.

mysql> SHOW DATABASES;
+--------------------+
| Database           |
+--------------------+
| information_schema | 
| mysql              | 
+--------------------+
2 rows in set (0.00 sec)

```

我们使用 mysql 监视器客户端应用连接到服务器。 我们使用根帐户连接到数据库。 我们用`SHOW DATABASES`语句显示所有可用的数据库。

```cs
mysql> CREATE DATABASE mydb;
Query OK, 1 row affected (0.02 sec)

```

我们创建一个新的`mydb`数据库。 在整个教程中，我们将使用此数据库。

```cs
mysql> CREATE USER user12@localhost IDENTIFIED BY '34klq*';
Query OK, 0 rows affected (0.00 sec)

mysql> USE mydb;
Database changed

mysql> GRANT ALL ON mydb.* to user12@localhost;
Query OK, 0 rows affected (0.00 sec)

mysql> quit;
Bye

```

我们创建一个新的数据库用户。 我们授予该用户`mydb`数据库所有表的所有特权。

## 定义

ADO.NET 是 .NET 框架的重要组成部分。 该规范统一了对关系数据库，XML 文件和其他应用数据的访问。MySQL Connector/Net 是 MySQL 数据库 ADO.NET 规范的实现。 它是用 C# 语言编写的驱动程序，可用于所有.NET 语言。

`Connection`，`Command`，`DataReader`，`DataSet`和`DataProvider`是 .NET 数据供应器模型的核心元素。 `Connection`创建到特定数据源的连接。 `Command`对象针对数据源执行一条 SQL 语句。 `DataReader`从数据源读取数据流。 `DataSet`对象用于脱机处理大量数据。 它是一种断开连接的数据表示形式，可以保存来自各种不同来源的数据。 `DataReader`和`DataSet`都用于处理数据。 它们在不同的情况下使用。 如果只需要读取查询结果，则`DataReader`是更好的选择。 如果需要更广泛的数据处理，或者要将 Winforms 控件绑定到数据库表，则首选`DataSet`。

## MySQL 版本

如果以下程序运行正常，则我们已安装一切正常。 我们检查 MySQL 服务器的版本。

```cs
using System;
using MySql.Data.MySqlClient; 

public class Example
{

    static void Main() 
    {
        string cs = @"server=localhost;userid=user12;
            password=34klq*;database=mydb";

        MySqlConnection conn = null;

        try 
        {
          conn = new MySqlConnection(cs);
          conn.Open();
          Console.WriteLine("MySQL version : {0}", conn.ServerVersion);

        } catch (MySqlException ex) 
        {
          Console.WriteLine("Error: {0}",  ex.ToString());

        } finally 
        {          
          if (conn != null) 
          {
              conn.Close();
          }
        }
    }
}

```

我们连接到数据库并获取有关 MySQL 服务器的一些信息。

```cs
using MySql.Data.MySqlClient; 

```

我们导入 MySQL 数据供应器的元素。

```cs
string cs = @"server=localhost;userid=user12;
    password=34klq*;database=mydb";

```

这是连接字符串。 数据提供者使用它来建立与数据库的连接。 我们指定主机名，用户名，密码和数据库名。

```cs
conn = new MySqlConnection(cs);

```

创建一个`MySQLConnection`对象。 该对象用于打开与数据库的连接。

```cs
conn.Open();

```

这行打开数据库连接。

```cs
Console.WriteLine("MySQL version : {0}", conn.ServerVersion);

```

在这里，我们使用连接对象的`ServerVersion`属性打印 MySQL 的版本。

```cs
} catch (MySqlException ex) 
{
  Console.WriteLine("Error: {0}",  ex.ToString());

```

如果发生异常，我们将错误消息打印到控制台。

```cs
} finally 
{  
  if (conn != null) 
  {
      conn.Close();
  }
}

```

在最后一步，我们关闭连接对象。

```cs
$ dmcs -r:/usr/lib/cli/MySql.Data-6.1/MySql.Data.dll version.cs 

```

我们汇编示例。 提供了 MySQL 连接器 DLL 的路径。

```cs
$ ./version.exe 
MySQL version : 5.5.9

```

这是我系统上程序的输出。

接下来是一个更复杂的程序。

```cs
using System;
using MySql.Data.MySqlClient; 

public class Example
{

    static void Main() 
    {
        string cs = @"server=localhost;userid=user12;
            password=34klq*;database=mydb";

        MySqlConnection conn = null;

        try 
        {
          conn = new MySqlConnection(cs);
          conn.Open();

          string stm = "SELECT VERSION()";   
          MySqlCommand cmd = new MySqlCommand(stm, conn);
          string version = Convert.ToString(cmd.ExecuteScalar());
          Console.WriteLine("MySQL version : {0}", version);

        } catch (MySqlException ex) 
        {
          Console.WriteLine("Error: {0}",  ex.ToString());

        } finally 
        {

          if (conn != null) 
          {
              conn.Close();
          }

        }
    }
}

```

我们检查 MySQL 数据库的版本。 这次使用 SQL 查询。

```cs
string stm = "SELECT VERSION()";

```

这是 SQL `SELECT`语句。 它返回数据库的版本。 `VERSION()`是内置的 MySQL 函数。

```cs
MySqlCommand cmd = new MySqlCommand(stm, conn);

```

`MySqlCommand`是一个对象，用于在数据库上执行查询。 参数是 SQL 语句和连接对象。

```cs
string version = Convert.ToString(cmd.ExecuteScalar());

```

有些查询仅返回标量值。 在我们的例子中，我们需要一个简单的字符串来指定数据库的版本。 在这种情况下使用`ExecuteScalar()`。 我们避免了使用更复杂的对象的开销。

```cs
$ ./version2.exe 
MySQL version : 5.5.9

```

结果与前面的示例相同。

## 创建和填充表

接下来，我们将创建数据库表并用数据填充它们。 这些表将在本教程中使用。

```cs
DROP TABLE IF EXISTS Books, Authors;

CREATE TABLE IF NOT EXISTS Authors(Id INT PRIMARY KEY AUTO_INCREMENT, 
    Name VARCHAR(25)) ENGINE=INNODB;

INSERT INTO Authors(Id, Name) VALUES(1, 'Jack London');
INSERT INTO Authors(Id, Name) VALUES(2, 'Honore de Balzac');
INSERT INTO Authors(Id, Name) VALUES(3, 'Lion Feuchtwanger');
INSERT INTO Authors(Id, Name) VALUES(4, 'Emile Zola');
INSERT INTO Authors(Id, Name) VALUES(5, 'Truman Capote');

CREATE TABLE IF NOT EXISTS Books(Id INT PRIMARY KEY AUTO_INCREMENT, 
    AuthorId INT, Title VARCHAR(100), 
    FOREIGN KEY(AuthorId) REFERENCES Authors(Id) ON DELETE CASCADE)
    ENGINE=INNODB;

INSERT INTO Books(Id, AuthorId, Title) VALUES(1, 1, 'Call of the Wild');
INSERT INTO Books(Id, AuthorId, Title) VALUES(2, 1, 'Martin Eden');
INSERT INTO Books(Id, AuthorId, Title) VALUES(3, 2, 'Old Goriot');
INSERT INTO Books(Id, AuthorId, Title) VALUES(4, 2, 'Cousin Bette');
INSERT INTO Books(Id, AuthorId, Title) VALUES(5, 3, 'Jew Suess');
INSERT INTO Books(Id, AuthorId, Title) VALUES(6, 4, 'Nana');
INSERT INTO Books(Id, AuthorId, Title) VALUES(7, 4, 'The Belly of Paris');
INSERT INTO Books(Id, AuthorId, Title) VALUES(8, 5, 'In Cold blood');
INSERT INTO Books(Id, AuthorId, Title) VALUES(9, 5, 'Breakfast at Tiffany');

```

我们有一个`books.sql`文件。 它创建两个数据库表：`Authors`和`Books`。 这些表是 InnoDB 类型的。 InnoDB 数据库支持外键约束和事务。 我们将外键约束放在`Books`表的`AuthorId`列上。 我们用初始数据填充表。

```cs
mysql> source books.sql
Query OK, 0 rows affected (0.07 sec)
Query OK, 0 rows affected (0.12 sec)
Query OK, 1 row affected (0.04 sec)
...

```

我们使用`source`命令执行`books.sql`脚本。

## 预备语句

现在，我们将以预备语句来关注自己。 在编写预备语句时，我们使用占位符，而不是直接将值写入语句中。 预准备的语句可提高安全性和性能。

```cs
using System;
using MySql.Data.MySqlClient; 

public class Example
{

    static void Main() 
    {
        string cs = @"server=localhost;userid=user12;
            password=34klq*;database=mydb";

        MySqlConnection conn = null;

        try 
        {
          conn = new MySqlConnection(cs);
          conn.Open();

          MySqlCommand cmd = new MySqlCommand();
          cmd.Connection = conn;
          cmd.CommandText = "INSERT INTO Authors(Name) VALUES(@Name)";
          cmd.Prepare();

          cmd.Parameters.AddWithValue("@Name", "Trygve Gulbranssen");
          cmd.ExecuteNonQuery();

        } catch (MySqlException ex) 
        {
          Console.WriteLine("Error: {0}",  ex.ToString());

        } finally 
        {
            if (conn != null) {
                conn.Close();
            }

        }
    }
}

```

我们将新作者添加到`Authors`表中。 我们使用参数化命令。

```cs
cmd.CommandText = "INSERT INTO Authors(Name) VALUES(@Name)";
cmd.Prepare();

```

在这里，我们创建一个预备语句。 在编写预备语句时，我们使用占位符，而不是直接将值写入语句中。 预备语句更快，并且可以防止 SQL 注入攻击。 `@Name`是一个占位符，稍后将填充。

```cs
cmd.Parameters.AddWithValue("@Name", "Trygve Gulbranssen");

```

值绑定到占位符。

```cs
cmd.ExecuteNonQuery();

```

执行预备语句。 当我们不希望返回任何数据时，我们使用`MySQLCommand`对象的`ExecuteNonQuery()`方法。 这是当我们创建数据库或执行`INSERT`，`UPDATE`和`DELETE`语句时。

```cs
$ ./prepared.exe 

mysql> SELECT * FROM Authors;
+----+--------------------+
| Id | Name               |
+----+--------------------+
|  1 | Jack London        |
|  2 | Honore de Balzac   |
|  3 | Lion Feuchtwanger  |
|  4 | Emile Zola         |
|  5 | Truman Capote      |
|  6 | Trygve Gulbranssen |
+----+--------------------+
6 rows in set (0.00 sec)

```

我们在表中插入了一位新作者。

## 使用`MySqlDataReader`检索数据

`MySqlDataReader`是用于从数据库检索数据的对象。 它提供对查询结果的快速，仅转发和只读访问。 这是从表中检索数据的最有效方法。

```cs
using System;
using MySql.Data.MySqlClient; 

public class Example
{

    static void Main() 
    {
        string cs = @"server=localhost;userid=user12;
            password=34klq*;database=mydb";

        MySqlConnection conn = null;
        MySqlDataReader rdr = null;

        try 
        {
            conn = new MySqlConnection(cs);
            conn.Open();

            string stm = "SELECT * FROM Authors";
            MySqlCommand cmd = new MySqlCommand(stm, conn);
            rdr = cmd.ExecuteReader();

            while (rdr.Read()) 
            {
                Console.WriteLine(rdr.GetInt32(0) + ": " 
                    + rdr.GetString(1));
            }

        } catch (MySqlException ex) 
        {
            Console.WriteLine("Error: {0}",  ex.ToString());

        } finally 
        {
            if (rdr != null) 
            {
                rdr.Close();
            }

            if (conn != null) 
            {
                conn.Close();
            }

        }
    }
}

```

我们从`Authors`表中获取所有作者并将其打印到控制台。

```cs
reader = cmd.ExecuteReader();

```

要创建`MySQLDataReader`，我们必须调用`MySqlCommand`对象的`ExecuteReader()`方法。

```cs
while (reader.Read()) 
{
    Console.WriteLine(reader.GetInt32(0) + ": " 
        + reader.GetString(1));
}

```

`Read()`方法将数据读取器移至下一条记录。 如果有更多行，则返回`true`；否则，返回`true`。 否则为假。 我们可以使用数组索引符号来检索值，或者使用特定的方法来访问其本机数据类型中的列值。 后者效率更高。

```cs
if (rdr != null) 
{
    rdr.Close();
}

```

阅读完毕后，请务必调用阅读器的`Close()`方法。

```cs
$ ./retrieve.exe 
1: Jack London
2: Honore de Balzac
3: Lion Feuchtwanger
4: Emile Zola
5: Truman Capote
6: Trygve Gulbranssen

```

这是示例的输出。

## 列标题

接下来，我们将展示如何使用数据库表中的数据打印列标题。

```cs
using System;
using MySql.Data.MySqlClient; 

public class Example
{

    static void Main() 
    {
        string cs = @"server=localhost;userid=user12;
            password=34klq*;database=mydb";

        MySqlConnection conn = null;
        MySqlDataReader rdr = null;

        try 
        {
            conn = new MySqlConnection(cs);
            conn.Open();

            string stm = @"SELECT Name, Title From Authors,
                Books WHERE Authors.Id=Books.AuthorId";

            MySqlCommand cmd = new MySqlCommand(stm, conn);
            rdr = cmd.ExecuteReader();

            Console.WriteLine("{0} {1}", rdr.GetName(0), 
                rdr.GetName(1).PadLeft(18));

            while (rdr.Read()) 
            {
                Console.WriteLine(rdr.GetString(0).PadRight(18) +
                    rdr.GetString(1));
            }

        } catch (MySqlException ex) 
        {
            Console.WriteLine("Error: {0}",  ex.ToString());

        } finally 
        {
            if (rdr != null) 
            {
                rdr.Close();
            }

            if (conn != null) 
            {
                conn.Close();
            }

        }
    }
}

```

在此程序中，我们从`Authors`表中选择作者，并从`Books`表中选择他们的书。

```cs
string stm = @"SELECT Name, Title From Authors,
    Books WHERE Authors.Id=Books.AuthorId";

```

这是将作者与他们的书联系在一起的 SQL 语句。

```cs
reader = cmd.ExecuteReader();

```

我们创建一个`MySqlDataReader`对象。

```cs
Console.WriteLine("{0} {1}", reader.GetName(0), 
    reader.GetName(1).PadLeft(18));

```

我们使用阅读器的`GetName()`方法获得列的名称。 `PadLeft()`方法返回指定长度的新字符串，其中当前字符串的开头用空格填充。 我们使用此方法正确对齐字符串。

```cs
while (reader.Read()) 
{
    Console.WriteLine(reader.GetString(0).PadRight(18) +
        reader.GetString(1));
}

```

我们将 SQL 语句返回的数据打印到终端。

```cs
$ ./headers.exe 
Name              Title
Jack London       Call of the Wild
Jack London       Martin Eden
Honore de Balzac  Old Goriot
Honore de Balzac  Cousin Bette
Lion Feuchtwanger Jew Suess
Emile Zola        Nana
Emile Zola        The Belly of Paris
Truman Capote     In Cold blood
Truman Capote     Breakfast at Tiffany

```

该程序的输出。

## 数据集& `MySqlDataAdapter`

`DataSet`是数据库表中数据的副本以及数据之间的关系。 它在内存中创建，并在需要对数据进行大量处理或将数据表绑定到 Winforms 控件时使用。 处理完成后，更改将被写入数据源。 `MySqlDataAdapter`是数据集和数据源之间的中介。 它填充`DataSet`并解析数据源的更新。

```cs
using System;
using System.Data;
using MySql.Data.MySqlClient; 

public class Example
{

    static void Main() 
    {
        string cs = @"server=localhost;userid=user12;
            password=34klq*;database=mydb";

        MySqlConnection conn = null;

        try 
        {
            conn = new MySqlConnection(cs);
            conn.Open();

            string stm = "SELECT * FROM Authors";
            MySqlDataAdapter da = new MySqlDataAdapter(stm, conn);

            DataSet ds = new DataSet();

            da.Fill(ds, "Authors");
            DataTable dt = ds.Tables["Authors"];

            dt.WriteXml("authors.xml");

            foreach (DataRow row in dt.Rows) 
            {            
                foreach (DataColumn col in dt.Columns) 
                {
                  Console.WriteLine(row[col]);
                }

                Console.WriteLine("".PadLeft(20, '='));
            }

        } catch (MySqlException ex) 
        {
            Console.WriteLine("Error: {0}",  ex.ToString());

        } finally 
        {          
            if (conn != null) 
            {
                conn.Close();
            }

        }
    }
}

```

我们从`Authors`表中打印作者。 我们还将它们保存在 XML 文件中。 这次，我们使用`MySqlDataAdapter`和`DataSet`对象。

```cs
MySqlDataAdapter da = new MySqlDataAdapter(stm, conn);

```

创建一个`MySqlDataAdapter`对象。 它以 SQL 语句和连接为参数。

```cs
DataSet ds = new DataSet();

da.Fill(ds, "Authors");

```

我们创建并填充`DataSet`。

```cs
DataTable dt = ds.Tables["Authors"];

```

我们得到名为“作者”的表。 我们只给了`DataSet`一个表，但是它可以包含多个表。

```cs
dt.WriteXml("authors.xml");

```

我们将数据写入 XML 文件。

```cs
foreach (DataRow row in dt.Rows) 
{            
    foreach (DataColumn col in dt.Columns) 
    {
      Console.WriteLine(row[col]);
    }

    Console.WriteLine("".PadLeft(20, '='));
}

```

我们向终端显示`Authors`表的内容。 为了遍历数据，我们利用了`DataTable`对象的行和列。

在下一个示例中，我们将表绑定到 Winforms `DataGrid`控件。

```cs
using System;
using System.Windows.Forms;
using System.Drawing;
using System.Data;
using MySql.Data.MySqlClient; 

class MForm : Form
{

    private DataGrid dg = null;
    private MySqlConnection conn = null;
    private MySqlDataAdapter da = null;        
    private DataSet ds = null;

    public MForm()
    {

       this.Text = "DataGrid";
       this.Size = new Size(350, 300);

       this.InitUI();
       this.InitData();

       this.CenterToScreen();
    }

    void InitUI()
    {    
        dg = new DataGrid();

        dg.CaptionBackColor = System.Drawing.Color.White;
        dg.CaptionForeColor = System.Drawing.Color.Black;
        dg.CaptionText = "Authors";

        dg.Location = new Point(8, 0);
        dg.Size = new Size(350, 300);
        dg.TabIndex = 0;
        dg.Parent = this;        
    }

    void InitData()
    {    
        string cs = @"server=localhost;userid=user12;
            password=34klq*;database=mydb";

        string stm = "SELECT * FROM Authors";

        try 
        {
            conn = new MySqlConnection(cs);          
            conn.Open();
            ds = new DataSet();
            da = new MySqlDataAdapter(stm, conn);
            da.Fill(ds, "Authors");  

            dg.DataSource = ds.Tables["Authors"];

        } catch (MySqlException ex) 
        {
            Console.WriteLine("Error: " + ex.ToString());

        } finally 
        {
            if (conn != null) 
            {
                conn.Close();
            }
        }    
    }
}

class MApplication 
{
    public static void Main() 
    {
        Application.Run(new MForm());
    }
}

```

在此示例中，我们将`Authors`表绑定到 Winforms `DataGrid`控件。

```cs
using System.Windows.Forms;
using System.Drawing;

```

这两个名称空间用于 GUI。

```cs
this.InitUI();
this.InitData();

```

在`InitUI()`方法内部，我们构建了用户界面。 在`InitData()`方法中，我们连接到数据库，将数据检索到`DataSet`中并将其绑定到`DataGrid`控件。

```cs
dg = new DataGrid();

```

`DataGrid`控件已创建。

```cs
string stm = "SELECT * FROM Authors";

```

我们将在`DataGrid`控件中显示`Authors`表中的数据。

```cs
dg.DataSource = ds.Tables["Authors"];

```

我们将`DataGrid`控件的`DataSource`属性绑定到所选表。

```cs
$ dmcs -r:/usr/lib/cli/MySql.Data-6.1/MySql.Data.dll -r:System.Windows.Forms.dll 
    -r:System.Drawing.dll -r:System.Data.dll dataadapter2.cs

```

要编译该示例，我们必须包括其他 DLL：用于 MySQL 连接器，`Winforms`，`Drawing`和`Data`的 DLL。

![DataGrid](img/01096affcc64a9fbfff831cb29a1c1e2.jpg)

图：`DataGrid`

## 事务支持

事务是针对一个或多个数据库中数据的数据库操作的基本单位。 事务中所有 SQL 语句的影响可以全部提交给数据库，也可以全部回滚。

MySQL 数据库具有不同类型的存储引擎。 最常见的是 MyISAM 和 InnoDB 引擎。 在数据安全性和数据库速度之间需要权衡。 MyISAM 表的处理速度更快，并且不支持事务。 另一方面，InnoDB 表可以更安全地防止数据丢失。 他们支持事务。 它们处理较慢。

```cs
using System;
using MySql.Data.MySqlClient; 

public class Example
{

    static void Main() 
    {
        string cs = @"server=localhost;userid=user12;
            password=34klq*;database=mydb";

        MySqlConnection conn = null;
        MySqlTransaction tr = null; 

        try 
        {
            conn = new MySqlConnection(cs); 
            conn.Open();
            tr = conn.BeginTransaction();

            MySqlCommand cmd = new MySqlCommand();
            cmd.Connection = conn;
            cmd.Transaction = tr;

            cmd.CommandText = "UPDATE Authors SET Name='Leo Tolstoy' WHERE Id=1";
            cmd.ExecuteNonQuery();
            cmd.CommandText = "UPDATE Books SET Title='War and Peace' WHERE Id=1";
            cmd.ExecuteNonQuery();
            cmd.CommandText = "UPDATE Books SET Titl='Anna Karenina' WHERE Id=2";
            cmd.ExecuteNonQuery();

            tr.Commit();

        } catch (MySqlException ex) 
        {
            try 
            { 
                tr.Rollback();                

            } catch (MySqlException ex1) 
            {
                Console.WriteLine("Error: {0}",  ex1.ToString());                
            }

            Console.WriteLine("Error: {0}",  ex.ToString());

        } finally 
        {
           if (conn != null)
           {
              conn.Close();
           }
        }
    }
}

```

在此程序中，我们想在`Authors`表的第一行上更改作者的姓名。 我们还必须更改与该作者相关的书籍。 一个需要进行事务的很好的例子。 如果我们更改作者但不更改作者的书，则数据已损坏。

```cs
MySqlTransaction tr = null; 

```

`MySqlTransaction`是用于处理事务的对象。

```cs
tr = conn.BeginTransaction();

```

我们开始事务。

```cs
cmd.CommandText = "UPDATE Books SET Titl='Anna Karenina' WHERE Id=2";
cmd.ExecuteNonQuery();

```

第三个 SQL 语句有一个错误。 表中没有“ Titl”列。

```cs
tr.Commit();

```

如果没有异常，则提交事务。

```cs
try 
{ 
    tr.Rollback();                

} catch (MySqlException ex1) 
{
    Console.WriteLine("Error: {0}",  ex1.ToString());                
}

```

发生异常时，事务将回滚。 没有更改提交到数据库。 在回滚期间，可能会出现`Exception`。 我们在单独的`try/catch`语句中对此进行处理。

```cs
$ ./transaction.exe 
Error: MySql.Data.MySqlClient.MySqlException: Unknown column 'Titl' in 'field list'
  at MySql.Data.MySqlClient.MySqlStream.ReadPacket () [0x00000] 
  at MySql.Data.MySqlClient.NativeDriver.ReadResult () [0x00000] 

mysql> SELECT Name, Title From Authors, Books WHERE Authors.Id=Books.AuthorId;
+-------------------+----------------------+
| Name              | Title                |
+-------------------+----------------------+
| Jack London       | Call of the Wild     |
| Jack London       | Martin Eden          |
| Honore de Balzac  | Old Goriot           |
| Honore de Balzac  | Cousin Bette         |
| Lion Feuchtwanger | Jew Suess            |
| Emile Zola        | Nana                 |
| Emile Zola        | The Belly of Paris   |
| Truman Capote     | In Cold blood        |
| Truman Capote     | Breakfast at Tiffany |
+-------------------+----------------------+
9 rows in set (0.00 sec)

```

引发异常。 事务已回滚，并且未进行任何更改。

但是，如果没有事务，数据是不安全的。

```cs
using System;
using MySql.Data.MySqlClient; 

public class Example
{

    static void Main() 
    {
        string cs = @"server=localhost;userid=user12;
            password=34klq*;database=mydb";

        MySqlConnection conn = null;

        try 
        {
            conn = new MySqlConnection(cs); 
            conn.Open();

            MySqlCommand cmd = new MySqlCommand();
            cmd.Connection = conn;

            cmd.CommandText = "UPDATE Authors SET Name='Leo Tolstoy' WHERE Id=1";
            cmd.ExecuteNonQuery();
            cmd.CommandText = "UPDATE Books SET Title='War and Peace' WHERE Id=1";
            cmd.ExecuteNonQuery();
            cmd.CommandText = "UPDATE Books SET Titl='Anna Karenina' WHERE Id=2";
            cmd.ExecuteNonQuery();

        } catch (MySqlException ex) 
        {
            Console.WriteLine("Error: {0}",  ex.ToString());

        } finally {
           if (conn != null)
           {
              conn.Close();
           }
        }
    }
}

```

我们有同样的例子。 这次，没有事务支持。

```cs
$ ./notransaction.exe 
Error: MySql.Data.MySqlClient.MySqlException: Unknown column 'Titl' in 'field list'
  at MySql.Data.MySqlClient.MySqlStream.ReadPacket () [0x00000] 
  at MySql.Data.MySqlClient.NativeDriver.ReadResult () [0x00000] 

mysql> SELECT Name, Title From Authors, Books WHERE Authors.Id=Books.AuthorId;
+-------------------+----------------------+
| Name              | Title                |
+-------------------+----------------------+
| Leo Tolstoy       | War and Peace        |
| Leo Tolstoy       | Martin Eden          |
| Honore de Balzac  | Old Goriot           |
| Honore de Balzac  | Cousin Bette         |
| Lion Feuchtwanger | Jew Suess            |
| Emile Zola        | Nana                 |
| Emile Zola        | The Belly of Paris   |
| Truman Capote     | In Cold blood        |
| Truman Capote     | Breakfast at Tiffany |
+-------------------+----------------------+
9 rows in set (0.00 sec)

```

再次引发异常。 列夫·托尔斯泰没有写马丁·伊甸园。 数据已损坏。

[Tweet](https://twitter.com/share) 

这是带有 MySQL 连接器的 MySQL C# 教程。 您可能也对 [MySQL C API 教程](/db/mysqlc/)， [MySQL Python 教程](/db/mysqlpython/)或 [MySQL PHP 教程](/databases/mysqlphptutorial/)感兴趣。