# PostgreSQL PHP 编程简介

> 原文： [http://zetcode.com/db/postgresqlphp/intro/](http://zetcode.com/db/postgresqlphp/intro/)

在 PostgreSQL PHP 教程的第一章中，我们将提供必要的定义。 我们将展示如何安装 PostgreSQL 数据库和所需的包。 这些示例将使用 PHP CLI 在命令行上运行。

为了运行示例，我们需要安装 PHP 语言（以 PHP CLI 的形式）和 PostgreSQL 数据库。 我们还需要`php5-psql`包。

## 关于 PostgreSQL 数据库

PostgreSQL 是一个功能强大的开源对象关系数据库系统。 它是一个多用户数据库管理系统。 它可以在多种平台上运行，包括 Linux，FreeBSD，Solaris，Microsoft Windows 和 Mac OS。 PostgreSQL 由 PostgreSQL 全球开发小组开发。

## 设置 PostgreSQL

我们必须安装 PostgreSQL 数据库。

```php
$ sudo apt-get install postgresql

```

在基于 Debian 的系统上，我们可以使用上述命令安装 PostgreSQL 数据库。

```php
$ sudo update-rc.d -f postgresql remove
 Removing any system startup links for /etc/init.d/postgresql ...
   /etc/rc0.d/K21postgresql
   /etc/rc1.d/K21postgresql
   /etc/rc2.d/S19postgresql
   /etc/rc3.d/S19postgresql
   /etc/rc4.d/S19postgresql
   /etc/rc5.d/S19postgresql
   /etc/rc6.d/K21postgresql

```

如果我们从包中安装 PostgreSQL 数据库，它将自动添加到操作系统的启动脚本中。 如果我们仅学习使用数据库，则不必在每次引导系统时都启动数据库。 上面的命令删除 PostgreSQL 数据库的所有系统启动链接。

```php
$ /etc/init.d/postgresql status
Running clusters: 9.1/main

$ service postgresql status
Running clusters: 9.1/main 

```

我们检查 PostgreSQL 服务器是否正在运行。 如果没有，我们需要启动服务器。

```php
$ sudo service postgresql start
 * Starting PostgreSQL 9.1 database server        [ OK ]

```

在 Ubuntu Linux 上，我们可以使用`service postgresql start`命令启动服务器。

```php
$ sudo service postgresql stop
[sudo] password for janbodnar: 
 * Stopping PostgreSQL 9.1 database server        [ OK ] 

```

我们使用`service postgresql stop`命令停止 PostgreSQL 服务器。

```php
$ sudo -u postgres createuser janbodnar
Shall the new role be a superuser? (y/n) n
Shall the new role be allowed to create databases? (y/n) y
Shall the new role be allowed to create more new roles? (y/n) n

```

我们在 PostgreSQL 系统中创建了一个新角色。 我们允许它具有创建新数据库的能力。 角色是数据库世界中的用户。 角色与操作系统用户是分开的。 我们创建了一个没有`-W`选项的新用户，例如我们尚未指定密码。 这使我们能够使用该用户连接到数据库，而无需密码验证。 请注意，这仅适用于本地主机。

```php
$ sudo -u postgres createdb testdb -O janbodnar

```

`createdb`命令使用所有者 janbodnar 创建一个新的 PostgreSQL 数据库。

## PHP CLI

已知 PHP 语言可在 Web 服务器上运行。 但是它也可以在命令行上使用。 PHP 命令行界面（PHP CLI）是一个库，使程序员可以在命令行上使用 PHP。 使用 PHP CLI，我们可以使用 PHP-GTK 构建 GUI 应用，也可以创建简单的测试脚本。 在本教程中，我们将使用命令行 PHP 解释器连接到 PostgreSQL 数据库。

```php
$ sudo apt-get install php5-cli

```

我们在 Linux 系统上安装 PHP CLI 模块。

## php5-pgsql 包

_php5-pgsql_ 是一个使用 PHP 语言与 PostgreSQL 数据库一起使用的包。 在其他系统上，包名称可能不同。

```php
$ sudo apt-get install php5-pgsql

```

我们启动以上命令来安装包。

## 处理错误

我们对处理 PHP 中的错误有一个简短的评论。 PHP 具有对错误报告的内置支持。 细节可以在 php.ini 文件中控制。 请注意，PHP CLI 版本具有单独的 INI 文件。 它位于我们系统上的`/etc/php5/cli/php.ini`中。

`display_errors`指令控制是否显示内置错误消息。 在开发环境中，将显示这些错误消息。 在生产中，它们被抑制了。 没有理由向用户显示这些技术消息。 另外，这是潜在的安全风险。

通常，我们应该将更具体的错误消息记录到日志文件中。 `log_errors`指令控制是否记录错误。 `error_log`指定应该记录脚本错误的文件的名称。 如果未设置，则默认为 PHP CLI 的`stderr`。

`pg_last_error()`函数获取连接的最后一条错误消息字符串。 它与内置错误报告中生成的错误消息相同。

在本教程的示例中，我们不使用`pg_last_error()`函数，因为它复制了内置错误消息。 我们有以下设置：

```php
...
display_errors = On
...
log_errors = On

; Our own custom based log file
error_log = /home/janbodnar/.phpcli_log
...

```

我们显示内置错误； 它们显示在命令行上。 错误消息还将记录到指定的日志文件中。 如果我们不想在控制台上显示错误消息，只需关闭`display_errors`指令即可。

```php
$rs = pg_query($con, $query) or die("Cannot execute query: $query\n");

```

在脚本中，我们使用`die()`函数显示一条简单的错误消息，该消息易于理解。 更具体的详细信息将保存到日志文件中。 `die()`功能也会终止脚本。

## 版本

在第一个代码示例中，我们将获取 PostgreSQL 数据库的版本。

```php
<?php 

$host = "localhost"; 
$user = "user12"; 
$pass = "34klq*"; 
$db = "testdb"; 

$con = pg_connect("host=$host dbname=$db user=$user password=$pass")
    or die ("Could not connect to server\n"); 

$query = "SELECT VERSION()"; 
$rs = pg_query($con, $query) or die("Cannot execute query: $query\n"); 
$row = pg_fetch_row($rs);

echo $row[0] . "\n";

pg_close($con); 

?>

```

在上面的 PHP 脚本中，我们连接到先前创建的`testdb`数据库。 我们执行一条 SQL 语句，该语句返回 PostgreSQL 数据库的版本。

```php
$host = "localhost"; 
$user = "user12"; 
$pass = "34klq*"; 
$db = "testdb"; 

```

这些是连接字符串变量。 要创建到 PostgreSQL 数据库的连接，我们必须提供主机名，用户名和密码以及数据库名。

```php
$con = pg_connect("host=$host dbname=$db user=$user password=$pass")
    or die ("Could not connect to server\n"); 

```

我们连接到数据库服务器。 如果无法创建连接，则`die()`函数将终止脚本并将错误消息打印到控制台。 `pg_connect()`函数返回连接资源，该资源将在以后与其他模块功能一起使用。

```php
$query = "SELECT VERSION()"; 

```

该 SQL 语句选择 PostgreSQL 数据库的版本。

```php
$rs = pg_query($con, $query) or die("Cannot execute query: $query\n"); 

```

使用`pg_query()`功能执行查询。

```php
$row = pg_fetch_row($rs);

```

我们从返回的结果中获取数据。

```php
echo $row[0] . "\n";

```

我们将检索到的数据打印到控制台。 数据以 PHP 数组的形式返回。 数组的第一个元素是我们要查找的字符串。

```php
pg_close($con); 

```

使用`pg_close()`功能关闭与数据库的连接。

```php
$ php version.php
PostgreSQL 9.1.3 on i686-pc-linux-gnu, compiled by gcc-4.6.real 
    (Ubuntu/Linaro 4.6.1-9ubuntu3) 4.6.1, 32-bit

```

运行`version.php`脚本。

## 插入数据

我们将创建一个`cars`表并在其中插入几行。

```php
<?php 

$host = "localhost"; 
$user = "user12"; 
$pass = "34klq*"; 
$db = "testdb"; 

$con = pg_connect("host=$host dbname=$db user=$user password=$pass")
    or die ("Could not connect to server\n"); 

$query = "DROP TABLE IF EXISTS cars"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

$query = "CREATE TABLE cars(id INTEGER PRIMARY KEY, mame VARCHAR(25), price INT)";  
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

$query = "INSERT INTO cars VALUES(1,'Audi',52642)"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

$query = "INSERT INTO cars VALUES(2,'Mercedes',57127)"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

$query = "INSERT INTO cars VALUES(3,'Skoda',9000)"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

$query = "INSERT INTO cars VALUES(4,'Volvo',29000)"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

$query = "INSERT INTO cars VALUES(5,'Bentley',350000)"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

$query = "INSERT INTO cars VALUES(6,'Citroen',21000)"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

$query = "INSERT INTO cars VALUES(7,'Hummer',41400)"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

$query = "INSERT INTO cars VALUES(8,'Volkswagen',21606)"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

pg_close($con); 

?>

```

上面的脚本创建一个`Cars`表，并将 8 行插入到该表中。

```php
$query = "DROP TABLE IF EXISTS cars"; 
pg_query($con, $query) or die("Cannot execute query: $query\n");

```

如果汽车表已经存在，我们将其删除。 `pg_query()`函数在指定的数据库连接上执行给定查询。

```php
$query = "CREATE TABLE cars(id INTEGER PRIMARY KEY, mame VARCHAR(25), price INT)"; 

```

该 SQL 语句创建一个新的 cars 表。 该表有三列。

```php
$query = "INSERT INTO cars VALUES(1,'Audi',52642)"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

$query = "INSERT INTO cars VALUES(2,'Mercedes',57127)"; 
pg_query($con, $query) or die("Cannot execute query: $query\n"); 

```

我们要把两辆车插入表。

```php
pg_close($con);

```

与数据库的连接已关闭。

```php
$ psql testdb
psql (9.1.3)
Type "help" for help.

testdb=# SELECT * FROM cars;
 id |    name    | price  
----+------------+--------
  1 | Audi       |  52642
  2 | Mercedes   |  57127
  3 | Skoda      |   9000
  4 | Volvo      |  29000
  5 | Bentley    | 350000
  6 | Citroen    |  21000
  7 | Hummer     |  41400
  8 | Volkswagen |  21606
  9 | BMW        |  36000
(9 rows)

```

我们使用`psql`工具验证写入的数据。

## 预备语句

现在，我们将以预备语句来关注自己。 在编写预备语句时，我们使用占位符，而不是直接将值写入语句中。 预准备的语句可提高安全性和性能。

预备语句是可用于优化性能的服务器端对象。 准备查询后，将对其进行解析，重写和计划。 以后只需要执行预备语句。 因此，解析，重写和计划阶段仅执行一次，而不是每次执行语句时。 准备的语句仅在当前数据库会话期间持续。 会话结束时，预备语句将被遗忘，因此必须在重新使用之前重新创建它。

```php
$host = "localhost"; 
$user = "user12"; 
$pass = "34klq*"; 
$db = "testdb"; 

$id = 9;
$name = "BMW";
$price = 36000;

$con = pg_connect("host=$host dbname=$db user=$user password=$pass")
    or die ("Could not connect to server\n"); 

$query = "INSERT INTO cars VALUES($1, $2, $3)"; 

pg_prepare($con, "prepare1", $query) 
    or die ("Cannot prepare statement\n"); 

pg_execute($con, "prepare1", array($id, $name, $price))
    or die ("Cannot execute statement\n"); 

echo "Row successfully inserted\n";

pg_close($con); 

```

我们向汽车表添加一行。 我们使用预备查询。

```php
$id = 9;
$name = "BMW";
$price = 36000;

```

我们有三个变量将用于构建查询。 这些值可以例如来自网络表单。

```php
$query = "INSERT INTO cars VALUES($1, $2, $3)"; 

```

这是带有`$1`，`$2`和`$3`占位符的 SQL 查询。 占位符将在以后填充。

```php
pg_prepare($con, "prepare1", $query) 
    or die ("Cannot prepare statement\n"); 

```

在这里，我们通过调用`pg_prepare()`函数准备查询。 函数的第二个参数是预备语句的名称。 每个连接必须唯一。 预备语句更快，并且可以防止 SQL 注入攻击。

```php
pg_execute($con, "prepare1", array($id, $name, $price))
    or die ("Cannot execute statement\n"); 

```

`pg_execute()`函数发送一个请求，以执行带有给定参数的预备语句，并等待结果。 这些值绑定到占位符。

```php
$ php prepared.php
Row successfully inserted

testdb=# SELECT * FROM cars;
 id |    name    | price  
----+------------+--------
  1 | Audi       |  52642
  2 | Mercedes   |  57127
  3 | Skoda      |   9000
  4 | Volvo      |  29000
  5 | Bentley    | 350000
  6 | Citroen    |  21000
  7 | Hummer     |  41400
  8 | Volkswagen |  21606
  9 | BMW        |  36000
(9 rows)

```

我们在表上放了一辆新车。

## 数据来源

创建本教程时，请参考 [PostgreSQL PHP 手册](http://www.php.net/manual/en/book.pgsql.php)和 [PostgreSQL 文档](http://www.postgresql.org/docs/9.1/interactive/index.html)。

这是 PostgreSQL PHP 教程的介绍性章节。