# Jython Swing 中的贪食蛇

> 原文： [http://zetcode.com/gui/jythonswing/nibbles/](http://zetcode.com/gui/jythonswing/nibbles/)

在 Jython Swing 编程教程的这一部分中，我们将创建一个贪食蛇游戏克隆。

贪食蛇是较旧的经典视频游戏。 它最初是在 70 年代后期创建的。 后来它被带到 PC 上。 在这个游戏中，玩家控制蛇。 目的是尽可能多地吃苹果。 蛇每次吃一个苹果，它的身体就会长大。 蛇必须避开墙壁和自己的身体。

## 开发

蛇的每个关节的大小为 10px。 蛇由光标键控制。 最初，蛇具有三个关节。 游戏立即开始。 游戏结束后，我们在窗口中心显示“`Game Over`”消息。

```py
import random

from java.awt import Color
from java.awt import Font
from java.awt import Toolkit
from java.awt.event import ActionListener
from java.awt.event import KeyEvent
from java.awt.event import KeyListener
from javax.swing import ImageIcon
from javax.swing import JPanel
from javax.swing import Timer

WIDTH = 300
HEIGHT = 300
DOT_SIZE = 10
ALL_DOTS = WIDTH * HEIGHT / (DOT_SIZE * DOT_SIZE)
RAND_POS = 29
DELAY = 140

x = [0] * ALL_DOTS
y = [0] * ALL_DOTS

class Board(JPanel, KeyListener, ActionListener):
    def __init__(self):
        super(Board, self).__init__()

        self.initUI()

    def initUI(self):

        self.setBackground(Color.black)

        iid = ImageIcon("dot.png")
        self.ball = iid.getImage()

        iia = ImageIcon("apple.png")
        self.apple = iia.getImage()

        iih = ImageIcon("head.png")
        self.head = iih.getImage()

        self.setFocusable(True)
        self.addKeyListener(self)
        self.initGame()

    def initGame(self):

        self.left = False
        self.right = True
        self.up = False
        self.down = False
        self.inGame = True
        self.dots = 3

        for i in range(self.dots):
            x[i] = 50 - i * 10
            y[i] = 50

        self.locateApple()

        self.timer = Timer(DELAY, self)
        self.timer.start()

    def paint(self, g):

        # due to bug, cannot call super()
        JPanel.paint(self, g)

        if self.inGame:
            self.drawObjects(g)

        else:
            self.gameOver(g)

    def drawObjects(self, g):

        g.drawImage(self.apple, self.apple_x, self.apple_y, self)

        for z in range(self.dots):
            if (z == 0):
                g.drawImage(self.head, x[z], y[z], self)
            else:
                g.drawImage(self.ball, x[z], y[z], self)

        Toolkit.getDefaultToolkit().sync()
        g.dispose()

    def gameOver(self, g):

        msg = "Game Over"
        small = Font("Helvetica", Font.BOLD, 14)
        metr = self.getFontMetrics(small)

        g.setColor(Color.white)
        g.setFont(small)
        g.drawString(msg, (WIDTH - metr.stringWidth(msg)) / 2,
                     HEIGHT / 2)

    def checkApple(self):

        if x[0] == self.apple_x and y[0] == self.apple_y:
            self.dots = self.dots + 1
            self.locateApple()

    def move(self):

        z = self.dots

        while z > 0:
            x[z] = x[(z - 1)]
            y[z] = y[(z - 1)]
            z = z - 1

        if self.left:
            x[0] -= DOT_SIZE

        if self.right:
            x[0] += DOT_SIZE

        if self.up:
            y[0] -= DOT_SIZE

        if self.down:
            y[0] += DOT_SIZE

    def checkCollision(self):

        z = self.dots

        while z > 0:
            if z > 4 and x[0] == x[z] and y[0] == y[z]:
                self.inGame = False
            z = z - 1

        if y[0] > HEIGHT - DOT_SIZE:
            self.inGame = False

        if y[0] < 0:
            self.inGame = False

        if x[0] > WIDTH - DOT_SIZE:
            self.inGame = False

        if x[0] < 0:
            self.inGame = False

    def locateApple(self):

        r = random.randint(0, RAND_POS)
        self.apple_x = r * DOT_SIZE
        r = random.randint(0, RAND_POS)
        self.apple_y = r * DOT_SIZE

#    public void actionPerformed(ActionEvent e) {

    def actionPerformed(self, e):

        if self.inGame:
            self.checkApple()
            self.checkCollision()
            self.move()
        else:
            self.timer.stop()

        self.repaint()

    def keyPressed(self, e):

        key = e.getKeyCode()

        if key == KeyEvent.VK_LEFT and not self.right:
            self.left = True
            self.up = False
            self.down = False

        if key == KeyEvent.VK_RIGHT and not self.left:
            self.right = True
            self.up = False
            self.down = False

        if key == KeyEvent.VK_UP and not self.down:
            self.up = True
            self.right = False
            self.left = False

        if key == KeyEvent.VK_DOWN and not self.up:
            self.down = True
            self.right = False
            self.left = False

```

首先，我们将定义一些在游戏中使用的常量。

`WIDTH`和`HEIGHT`常数确定电路板的大小。 `DOT_SIZE`是苹果的大小和蛇的点。 `ALL_DOTS`常数定义了板上可能的最大点数。 `RAND_POS`常数用于计算苹果的随机位置。 `DELAY`常数确定游戏的速度。

```py
x = [0] * ALL_DOTS
y = [0] * ALL_DOTS

```

这两个数组存储蛇的所有可能关节的 x，y 坐标。

`initGame()`方法初始化变量，加载图像并启动超时功能。

```py
def paint(self, g):

    JPanel.paint(self, g)

    if self.inGame:
        self.drawObjects(g)

    else:
        self.gameOver(g)

```

在`paint()`方法内部，我们检查`inGame`变量。 如果为真，则绘制对象。 苹果和蛇的关节。 否则，我们显示“游戏结束”文本。

```py
def drawObjects(self, g):

    g.drawImage(self.apple, self.apple_x, self.apple_y, self)

    for z in range(self.dots):
        if (z == 0):
            g.drawImage(self.head, x[z], y[z], self)
        else:
            g.drawImage(self.ball, x[z], y[z], self)

    Toolkit.getDefaultToolkit().sync()
    g.dispose()

```

`drawObjects()`方法绘制苹果和蛇的关节。 蛇的第一个关节是其头部，用红色圆圈表示。 `Toolkit.getDefaultToolkit().sync()`方法可确保显示为最新。 这对于动画很有用。

```py
def checkApple(self):

    if x[0] == self.apple_x and y[0] == self.apple_y:
        self.dots = self.dots + 1
        self.locateApple()

```

`checkApple()`方法检查蛇是否击中了苹果对象。 如果是这样，我们添加另一个蛇形关节并调用`locateApple()`方法，该方法将随机放置一个新的`Apple`对象。

在`move()`方法中，我们有游戏的密钥算法。 要了解它，请看一下蛇是如何运动的。 您控制蛇的头。 您可以使用光标键更改其方向。 其余关节在链上向上移动一个位置。 第二关节移动到第一个关节的位置，第三关节移动到第二个关节的位置，依此类推。

```py
while z > 0:
    x[z] = x[(z - 1)]
    y[z] = y[(z - 1)]
    z = z - 1

```

该代码将关节向上移动。

```py
if self.left:
    x[0] -= DOT_SIZE

```

将头向左移动。

在`checkCollision()`方法中，我们确定蛇是否击中了自己或撞墙之一。

```py
while z > 0:
    if z > 4 and x[0] == x[z] and y[0] == y[z]:
        self.inGame = False
    z = z - 1

```

如果蛇用头撞到关节之一，我们就结束游戏。

```py
if y[0] > HEIGHT - DOT_SIZE:
    self.inGame = False

```

如果蛇击中了棋盘的底部，我们就结束了游戏。

`locateApple()`方法在板上随机放置一个苹果。

```py
r = random.randint(0, RAND_POS)

```

我们得到一个从 0 到`RAND_POS-1`的随机数。

```py
self.apple_x = r * DOT_SIZE
...
self.apple_y = r * DOT_SIZE

```

这些行设置了`apple`对象的 x，y 坐标。

```py
def actionPerformed(self, e):

    if self.inGame:
        self.checkApple()
        self.checkCollision()
        self.move()
    else:
        self.timer.stop()

    self.repaint()

```

每隔`DELAY` ms，将调用`actionPerformed()`方法。 如果我们参与了游戏，我们将调用三种构建游戏逻辑的方法。 否则，我们将停止计时器。

在`Board`类的`keyPressed()`方法中，我们确定按下的键。

```py
if key == KeyEvent.VK_LEFT and not self.right:
    self.left = True
    self.up = False
    self.down = False

```

如果单击左光标键，则将`left`变量设置为`true`。 在`move()`方法中使用此变量来更改蛇对象的坐标。 还要注意，当蛇向右行驶时，我们不能立即向左转。

```py
#!/usr/local/bin/jython
# -*- coding: utf-8 -*-

"""
ZetCode Jython Swing tutorial

This is a simple Nibbles game
clone.

author: Jan Bodnar
website: zetcode.com
last edited: December 2010
"""

from java.awt import Dimension
from javax.swing import JFrame
from Board import Board

class Nibbles(JFrame):
    def __init__(self):
        super(Nibbles, self).__init__()

        self.initUI()

    def initUI(self):

        self.board = Board()
        self.board.setPreferredSize(Dimension(300, 300))
        self.add(self.board)

        self.setTitle("Nibbles")

        self.pack()
        self.setResizable(False)
        self.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE)
        self.setLocationRelativeTo(None)
        self.setVisible(True)

if __name__ == '__main__':
    Nibbles()

```

在本课程中，我们设置了半字节游戏。

![Nibbles](img/6cbd152194890384d5ac4820b904749e.jpg)

图：贪食蛇

这是使用 Swing 库和 Jython 编程语言编写的贪食蛇电脑游戏。