# Java `int`到`String`的转换

原文：http://zetcode.com/java/inttostring/

Java `int`到`String`教程展示了如何将整数转换为字符串。 有几种方法可以在 Java 中执行从`int`到`String`的转换。 我们可以使用字符串连接，字符串格式化，字符串构建以及内置的转换方法。

## 整数到字符串的转换

整数到字符串的转换是类型转换或类型转换，其中整数数据类型的实体更改为字符串一。

在本教程的示例中，我们构建了一个包含整数的字符串消息。

## 使用`String.format()`将`int`转换为`String`

`String.format()`使用指定的格式字符串和参数返回格式化的字符串。

`JavaIntToString.java`

```java
package com.zetcode;

public class JavaIntToString {

    public static void main(String[] args) {

        int numOfApples = 16;

        String msg = String.format("There are %s apples", numOfApples);

        System.out.println(msg);             
    }
}

```

该示例使用`String.format()`进行从`int`到`String`的转换。

## 使用字符串连接将`int`转换为`String`

当我们在`int`和`String`参数上使用`+`运算符时，Java 编译器在内部执行类型转换。 请注意，许多 Java 程序员都不喜欢这种转换。

`JavaIntToString2.java`

```java
package com.zetcode;

public class JavaIntToString2 {

    public static void main(String[] args) {

        int numOfApples = 16;

        String msg = "There are " + numOfApples + " apples";

        System.out.println(msg);   
    }
}

```

该示例使用字符串连接将 int 转换为 String。 在内部，Java 编译器使用`StringBuilder`进行转换。

## 使用`Integer.toString()`将`int`转换为`String`

`Integer.toString()`将其参数转换为带符号的十进制表示形式，并以字符串形式返回。

`JavaIntToString3.java`

```java
package com.zetcode;

public class JavaIntToString3 {

    public static void main(String[] args) {

        int numOfApples = 16;

        String msg = "There are " + Integer.toString(numOfApples) + " apples";

        System.out.println(msg);          
    }
}

```

该示例使用`Integer.toString()`进行从`int`到`String`的转换。

## 使用`String.valueOf()`将`int`转换为`String`

`String.valueOf()`返回整数参数的字符串表示形式。 使用`String.valueOf()`时，Java 编译器内部调用`Integer.toString()`。

`String.valueOf()`被许多人认为是最易读的方法。

`JavaIntToString4.java`

```java
package com.zetcode;

public class JavaIntToString4 {

    public static void main(String[] args) {

        int nOfApples = 16;

        String msg = "There are " + String.valueOf(nOfApples) + " apples";

        System.out.println(msg);              
    }
}

```

该示例使用`String.valueOf()`进行从`int`到`String`的转换。

## 使用`StringBuilder`将`int`转换为`String`

`StringBuilder`表示可变的字符串。 我们可以使用`StringBuilder`构造字符串。 我们也可以将整数附加到生成器。

`JavaIntToString5.java`

```java
package com.zetcode;

public class JavaIntToString5 {

    public static void main(String[] args) {

        int numOfApples = 16;

        StringBuilder msg = new StringBuilder();
        msg.append("There are ").append(numOfApples).append(" apples");

        System.out.println(msg);          
    }
}

```

该代码示例使用`StringBuilder`进行从`int`到`String`的转换。

## Java `int`到`String`的示例

以下示例使用`Scanner`从用户读取一个整数。 然后，将一条字符串消息打印到控制台，其中包含用户的输入。

`JavaIntToStringEx.java`

```java
package com.zetcode;

import java.util.Scanner;

public class JavaIntToStringEx {

    public static void main(String[] args) {

        System.out.print("Enter number of apples: ");

        try (Scanner scan = new Scanner(System.in)) {

            int numOfApples = scan.nextInt();

            String msg = String.format("There are %d apples", numOfApples);

            System.out.println(msg);
        }
    }
}

```

我们使用`String.format()`将`int`转换为`String`。

```java
try (Scanner scan = new Scanner(System.in)) {

```

`Scanner`是一个简单的文本扫描程序，可以使用正则表达式解析原始类型和字符串。 我们使用`Scanner`来读取用户的输入。

```java
int numOfApples = scan.nextInt();

```

`nextInt()`方法将输入的下一个标记扫描为`int`。

```java
String msg = String.format("There are %d apples", numOfApples);

```

使用`String.format()`创建一条消息。 它以用户输入作为第二个参数。

在本教程中，我们展示了如何在 Java 中将整数转换为字符串。 您可能也对相关教程感兴趣： [Java `StringBuilder`教程](/java/stringbuilder/)， [Java `HashSet`教程](/java/hashset/)， [Java `HashMap`教程](/java/hashmap/)， [Java 静态关键字](/java/statickeyword/)， [Java 中的 `HashMap`迭代](/java/hashmapiterate/)， [Java8 `forEach`教程](/articles/java8foreach/)，[读取 Java 中的文本文件](/articles/javareadtext/)，[用 Java 读取和写入 ICO 图像](/articles/javaico/)， [Java 教程](/lang/java/)，[用 Java 显示图像](/java/displayimage/)。