# Python 语法结构

> 原文： [http://zetcode.com/lang/python/lexicalstructure/](http://zetcode.com/lang/python/lexicalstructure/)

像人类语言一样，计算机语言也具有语法结构。 Python 程序的源代码由令牌组成。 令牌是原子代码元素。 在 Python 语言中，我们具有注释，变量，字面值，运算符，定界符和关键字。

## Python 注释

注释被人类用来阐明源代码。 Python 语言中的所有注释均以# 字符为准。

`comments.py`

```py
#!/usr/bin/env python

# comments.py
# Author Jan Bodnar
# ZetCode 2017

def main():
    print("Comments example")

main()

```

Python 解释器将忽略# 字符后的所有内容。

## Python 变量

变量是保存值的标识符。 在编程中，我们说我们为变量分配了一个值。 从技术上讲，变量是对存储值的计算机内存的引用。 在 Python 语言中，变量可以包含字符串，数字或各种对象，例如函数或类。 可以随时间为变量分配不同的值。

Python 中的变量可以由字母数字字符和下划线`_`字符创建。 变量不能以数字开头。 这样，Python 解释器可以更轻松地区分数字和变量。

```py
Value
value2
company_name

```

这些是有效的标识符。

```py
12Val
exx$
first-name

```

这些是无效标识符的示例。

变量为区分大小写。 这意味着`Price`，`price`和`PRICE`是三个不同的标识符。

`case.py`

```py
#!/usr/bin/env python

number = 10
Number = 11
NUMBER = 12

print(number, Number, NUMBER)

```

在脚本中，我们将三个数值分配给三个标识符。 虽然可以定义大小写不同的三个变量，但是这并不是一个好的编程习惯。

```py
$ ./case.py
10 11 12

```

这是脚本的输出。

## Python 字面值

字面值是表示 Python 源代码中的值的任何符号。 从技术上讲，字面值是在编译时分配一个值的，而变量是在运行时分配的。

```py
age = 29
nationality = "Hungarian"

```

在这里，我们为变量分配了两个字面值。 数字`29`和字符串`"Hungarian"`是字面值。

`literals.py`

```py
#!/usr/bin/env python

# literals.py

name1 = "Jane"
age1 = 12

name2 = "Rose"
age2 = 16

"Patrick"
34

"Luke"
23

print(name1, age1)
print(name2, age2)

```

如果我们不给变量分配字面值，就无法使用它。 它被丢弃。

```py
$ ./literals.py
Jane 12
Rose 16

```

这是`literals.py`脚本的输出。

## Python 运算符

运算符是用于对某个值执行操作的符号。

```py
+    -    ~    *    **    /    //
%    <<    >>    &    |    ^
and    or    not    in    not in
is    is not    <   >    !=
==    <=    >=

```

这是 Python 语言中可用的运算符的列表。 我们将在本教程的后面部分讨论运算符。

## Python 缩进

缩进用于在 Python 中定界。 在其他编程语言使用大括号或诸如`begin`，`end`之类的关键字的情况下，Python 使用空格。 某些语句后缩进增加； 缩进的减少表示当前块的结尾。 Python 样式指南建议每个缩进级别使用四个空格。

```py
if age > 18:
    print("adult person")

for i in range(5):
    print(i)

```

在`if`关键字之后，需要一个代码块。 新的语句在新行上开始，并以四个空格字符缩进。 随后的`for`关键字的缩进减少到最初的关键字。 `for`关键字启动一个新的代码块，在该代码块中缩进其语句。

## Python 分隔符

定界符是一个或多个字符的序列，用于指定纯文本或其他数据流中单独的独立区域之间的边界。

```py
(       )       [       ]       {       }
,       :       .       `       =       ;
+=      -=      *=      /=      //=     %=
<=      |=      ^=      >>=     <<=     **=
'       "       \       @

```

分隔符用于 Python 语言的各个领域。 它们用于构建表达式，字符串字面值，元组，字典或列表。

## Python 关键字

关键字是 Python 编程语言中的保留字。 关键字用于在计算机程序中执行特定任务。 例如，导入其他代码，执行重复性任务或执行逻辑操作。 程序员不能将关键字用作普通变量。

```py
and       del       global      not      with
as        elif      if          or       yield
assert    else      import      pass     False
break     except    in          raise    None
class     finally   is          return   True
continue  for       lambda      try
def       from      nonlocal    while

```

这是 Python 关键字的列表。 在本教程中，我们专门为关键字专门章节。

这就是 Python 的语法结构。