# Java servlet 发送图像教程

> 原文： [http://zetcode.com/articles/javaservletimage/](http://zetcode.com/articles/javaservletimage/)

在 Java servlet 发送图像教程中，我们使用 servlet 在 Java 中创建经典的 Web 应用。 Servlet 将图像发送到客户端。 Web 应用已部署在 Jetty 服务器上。

## Java Servlet

Servlet 是 Java 类，可响应特定类型的网络请求-最常见的是 HTTP 请求。 Servlet 用于实现 Web 应用。 它们在 Tomcat 或 Jetty 之类的 servlet 容器中运行。 在现代 Java Web 开发中，程序员使用在 Servlet 之上构建的框架。

Eclipse Jetty 是 Java HTTP 服务器和 Java Servlet 容器。 Jetty 可以轻松地嵌入到设备，工具，框架，应用服务器和群集中。

## Java servlet 图像示例

以下 Web 应用将图像发送到客户端。 该 Web 应用使用 Java Servlet。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>sendimageservlet</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>war</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>12</maven.compiler.source>
        <maven.compiler.target>12</maven.compiler.target>
    </properties>

    <dependencies>
        <dependency>
            <groupId>javax.servlet</groupId>
            <artifactId>javax.servlet-api</artifactId>
            <version>4.0.1</version>
            <scope>provided</scope>
        </dependency>

    </dependencies>

    <build>
        <plugins>

            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-war-plugin</artifactId>
                <version>3.2.2</version>
            </plugin>

            <plugin>
                <groupId>org.eclipse.jetty</groupId>
                <artifactId>jetty-maven-plugin</artifactId>
                <version>9.4.14.v20181114</version>
            </plugin>
        </plugins>
    </build>

</project>

```

`javax.servlet-api`依赖性是用于构建 Java Servlet 的库。 `maven-war-plugin`收集 Web 应用的所有工件依赖项，类和资源，并将它们打包到 Web 应用存档（WAR）中。 `jetty-maven-plugin`插件对于使用 Jetty 服务器进行快速开发和测试非常有用。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │               SendImageServlet.java
│   ├───resources
│   └───webapp
│       │   index.html
│       └───images
│               sid.jpg
└───test
    └───java

```

这是项目目录结构。

`webapp/index.html`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Servlet image</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
    </head>
    <body>
        <a href="image">Get image</a>
    </body>
</html>

```

`index.html`文件是我们应用的主页。 它具有一个链接，该链接调用一个 servlet 来管理图像文件。

`com/zetcode/SendImageServlet.java`

```java
package com.zetcode;

import javax.servlet.ServletContext;
import javax.servlet.annotation.WebServlet;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

@WebServlet(name = "SendImageServlet", urlPatterns = {"/image"})
public class SendImageServlet extends HttpServlet {

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws IOException {

        ServletContext sc = getServletContext();

        try (InputStream is = sc.getResourceAsStream(&quoimg/sid.jpg")) {

            // it is the responsibility of the container to close output stream
            OutputStream os = response.getOutputStream();

            if (is == null) {

                response.setContentType("text/plain");
                os.write("Failed to send image".getBytes());
            } else {

                byte[] buffer = new byte[1024];
                int bytesRead;

                response.setContentType("image/png");

                while ((bytesRead = is.read(buffer)) != -1) {

                    os.write(buffer, 0, bytesRead);
                }
            }
        }
    }
}

```

`SendImageServlet` Servlet 将图像文件返回给客户端。

```java
@WebServlet(name = "SendImageServlet", urlPatterns = {"/image"})

```

`@WebServlet`批注将具有`image` URL 模式的请求映射到`SendImageServlet` servlet。

```java
@Override
protected void doGet(HttpServletRequest request, HttpServletResponse response)
        throws IOException {

```

该请求是一个 GET 请求，因此我们以`doGet()`方法为其提供服务。

```java
ServletContext sc = getServletContext();

```

我们得到`ServletContext`，其中包含 servlet 用于与其 servlet 容器进行通信的一组方法，例如，获取文件的 MIME 类型，调度请求或写入日志文件。

```java
try (InputStream is = sc.getResourceAsStream(&quoimg/sid.jpg")) {

```

我们使用`getResourceAsStream()`获得图像资源流。

```java
OutputStream os = response.getOutputStream();

```

我们得到 servlet 输出流。 我们将图像数据写入此流。 容器负责关闭 servlet 输出流。

```java
if (is == null) {

    response.setContentType("text/plain");
    os.write("Failed to send image".getBytes());
} else {

```

如果无法打开图像输入流，则会将错误消息发送回客户端。

```java
response.setContentType("image/png");

```

图片为 PNG 格式； 因此，我们将响应的内容类型设置为`image/png`。

```java
byte[] buffer = new byte[1024];
int bytesRead;

response.setContentType("image/png");

while ((bytesRead = is.read(buffer)) != -1) {

    os.write(buffer, 0, bytesRead);
}

```

如果成功打开了图像输入流，我们将读取数据并将其写入 servlet 输出流。 我们将响应内容类型设置为`image/png`。

```java
$ mvn jetty:run

```

我们运行 Jetty 服务器并导航到`localhost:8080`。

在 Java servlet 发送图像教程中，我们使用了 Java servlet 将图像发送到客户端。

您可能也对以下相关教程感兴趣： [Java Servlet 复选框教程](/articles/javaservletcheckbox/)， [Java Servlet PDF 教程](/articles/javaservletpdf/)， [Java Servlet 图表教程](/articles/javaservletchart/)， [Servlet FreeMarker `JdbcTemplate`教程](/articles/servletfreemarker/)，[在 Spring Boot 中提供图像文件](/articles/springbootserveimage/)， [Java 教程](/lang/java/)或 [jQuery DatePicker教程](/articles/jquerydatepicker/)。

列出 [Java Servlet 教程](/all/#servlets)。