# Java `ServletConfig`教程

> 原文： [http://zetcode.com/articles/javaservletconfig/](http://zetcode.com/articles/javaservletconfig/)

Java `ServletConfig`教程展示了如何使用`ServletConfig`将初始化数据传递给 Servlet。

`ServletConfig`是 servlet 容器使用的 servlet 配置对象，用于在初始化期间将信息传递给 servlet。 Servlet 容器为 Web 应用中的每个 Servlet 创建一个`ServletConfig`。

## Java Servlet

Servlet 是 Java 类，可响应特定类型的网络请求-最常见的是 HTTP 请求。 Java servlet 用于创建 Web 应用。 它们在 servlet 容器（例如 Tomcat 或 Jetty）中运行。 现代 Java Web 开发使用在 servlet 之上构建的框架。

## Pure.css

Pure.css 是一组小型的响应式 CSS 模块，可以在每个 Web 项目中使用。 该库是由 Yahoo 创建的。

## Java `ServletConfig`示例

在以下 Web 应用中，我们有一个简单的 Web 表单。 我们向 Servlet 发送了一个名称参数。 如果参数为空，则通过`ServletConfig`读取初始化参数。 在示例中，我们还使用 Yahoo 的 Pure.css 库。

```java
$ tree
.
├── nb-configuration.xml
├── pom.xml
└── src
    ├── main
    │   ├── java
    │   │   └── com
    │   │       └── zetcode
    │   │           └── web
    │   │               └── MyServlet.java
    │   └── webapp
    │       ├── index.html
    │       ├── META-INF
    │       │   └── context.xml
    │       └── WEB-INF
    └── test
        └── java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0" 
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 
http://maven.apache.org/xsd/maven-4.0.0.xsd">

    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>JavaServletConfigEx</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>war</packaging>

    <name>JavaServletConfigEx</name>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>1.8</maven.compiler.source>
        <maven.compiler.target>1.8</maven.compiler.target>
    </properties>

    <dependencies>
        <dependency>
            <groupId>javax.servlet</groupId>
            <artifactId>javax.servlet-api</artifactId>
            <version>3.1.0</version>
            <scope>provided</scope>
        </dependency>
    </dependencies>

    <build>
        <plugins>

            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-war-plugin</artifactId>
                <version>2.3</version>
                <configuration>
                    <failOnMissingWebXml>false</failOnMissingWebXml>
                </configuration>
            </plugin>

        </plugins>
    </build>

</project>

```

这是 Maven `pom.xml`文件。 `javax.servlet-api`工件用于 Java Servlet。 `maven-war-plugin`负责收集 Web 应用的所有工件依赖项，类和资源，并将它们打包到 Web 应用存档（WAR）中。

`context.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<Context path="/JavaServletConfigEx"/>

```

在 Tomcat `context.xml`文件中，我们定义了上下文路径。 它是 Web 应用的名称。

`index.html`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Home Page</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1">
        <link rel="stylesheet" href="https://unpkg.com/purecss@1.0.0/build/pure-min.css">
    </head>
    <body>
        <form class="pure-form" action="MyServlet">
            <fieldset>
                <legend>Enter your name</legend>

                <input type="text" name="name">
                <button type="submit" class="pure-button pure-button-primary">Submit</button>
            </fieldset>
        </form>
    </body>
</html>

```

这是主页。 它包含一个`HTML`表格。 提交表单后，处理将发送到`MyServlet`。

```java
<link rel="stylesheet" href="https://unpkg.com/purecss@1.0.0/build/pure-min.css">

```

我们包括 Pure.css 库。

```java
<form class="pure-form" action="MyServlet">

```

`form`标记使用 Pure.css 库中的`pure-form`类。 `action`属性指向`MyServlet`。

```java
<input type="text" name="name">

```

用户输入的名称值将作为`name`参数发送到 servlet。

```java
<button type="submit" class="pure-button pure-button-primary">Submit</button>

```

提交按钮使用`pure-button`和`pure-button-primary`类。

`MyServlet.java`

```java
package com.zetcode.web;

import java.io.IOException;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletOutputStream;
import javax.servlet.annotation.WebServlet;
import javax.servlet.annotation.WebInitParam;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@WebServlet(name = "MyServlet", urlPatterns = {"/MyServlet"}, initParams = {
    @WebInitParam(name = "name", value = "Guest")})
public class MyServlet extends HttpServlet {

    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, IOException {

        response.setContentType("text/plain;charset=UTF-8");

        String name = request.getParameter("name");
        name = name.trim();

        if (name.isEmpty()) {

            ServletConfig sc = getServletConfig();

            name = sc.getInitParameter("name");
        }

        ServletOutputStream os = response.getOutputStream();
        os.println("Hello " + name);
    }
}

```

`MyServlet`从请求中读取`name`属性并生成输出。 输出为纯文本。

```java
@WebServlet(name = "MyServlet", urlPatterns = {"/MyServlet"}, initParams = {
    @WebInitParam(name = "name", value = "Guest")})

```

使用`@WebInitParam`，将`name`参数初始化为“来宾”值。

```java
String name = request.getParameter("name");
name = name.trim();

```

我们从请求对象和修剪空间读取`name`参数。

```java
if (name.isEmpty()) {

    ServletConfig sc = getServletConfig();

    name = sc.getInitParameter("name");
}

```

如果用户未输入任何值，我们将使用`ServletConfig`读取`name`初始化参数。 使用`getServletConfig()`检索`ServletConfig`。 使用`getInitParameter()`检索参数。

```java
ServletOutputStream os = response.getOutputStream();
os.println("Hello " + name);

```

我们将文本消息写入`ServletOutputStream`。

在本教程中，我们使用`ServletConfig`来读取 Java Servlet 中的初始化参数。

您可能也对以下相关教程感兴趣： [Java Servlet RESTful 客户端](/articles/javaservletrestclient/)， [Java Servlet PDF 教程](/articles/javaservletpdf/)， [Java `RequestDispatcher`](/java/requestdispatcher/) ，[从 servlet 提供纯文本](/articles/javaservlettext/) ， [Java Servlet 上传文件](/articles/javaservletuploadfile/)， [Java servlet 图像教程](/articles/javaservletimage/)， [Java Servlet HTTP 标头](/articles/javaservlethttpheaders/)或 [Java 教程](/lang/java/)