# Spring Boot Data JPA `@Query`教程

> 原文： [http://zetcode.com/springboot/datajpaquery/](http://zetcode.com/springboot/datajpaquery/)

Spring Boot Data JPA `@Query`教程展示了如何使用 Data JPA `@Query`创建自定义查询。

Spring 是用于创建企业应用的流行 Java 应用框架。 Spring Boot 是 Spring 框架的演进，可帮助您轻松创建独立的，生产级的基于 Spring 的应用。

## Spring Data JPA

Spring Data JPA 有助于实现基于 JPA 的存储库。 它增强了对基于 JPA 的数据访问层的支持。 它使构建使用数据访问技术的 Spring 支持的应用变得更加容易。 Spring Data JPA 是较大的 Spring Data 系列的一部分。

## Spring Data JPA `@Query`

`@Query`注解直接在存储库方法上声明查找程序查询。 虽然在域类上使用了类似的`@NamedQuery`，但在`Repository`接口上使用了 Spring Data JPA `@Query`注解。 这样可以将域类从持久性特定的信息中解放出来，这是一件好事。

## Spring Boot Data JPA `@Query`示例

以下应用是一个简单的 Spring Boot Web 应用，它使用 Data JPA @Query 创建一个自定义查询。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           ├───controller
│   │           │       MyController.java
│   │           ├───model
│   │           │       City.java
│   │           ├───repository
│   │           │       CityRepository.java
│   │           └───service
│   │                   CityService.java
│   │                   ICityService.java
│   └───resources
│       │   application.properties
│       │   data-h2.sql
│       │   schema-h2.sql
│       ├───static
│       │       index.html
│       └───templates
│               showCities.ftl
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springbootcustomdatajpaquery</artifactId>
    <version>1.0-SNAPSHOT</version>

    <packaging>jar</packaging>
    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>

        <dependency>
            <groupId>com.h2database</groupId>
            <artifactId>h2</artifactId>
            <scope>runtime</scope>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-web</artifactId>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-freemarker</artifactId>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-data-jpa</artifactId>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>

</project>

```

Maven POM 文件包含 H2 数据库，Freemarker 模板引擎和 Spring Boot Data JPA 的依赖项。

`resources/application.properties`

```java
server.servlet.contextPath=/myapp

spring.main.banner-mode=off
spring.datasource.platform=h2
logging.level.org.hibernate.SQL=DEBUG
spring.jpa.show-sql=true
spring.jpa.hibernate.ddl-auto=none

```

在`application.properties`文件中，我们编写了 Spring Boot 应用的各种配置设置。 `server.servlet.contextPath`设置上下文路径（应用名称）。 完成这些设置后，我们可以通过`localhost:8080/myapp/`访问该应用。 使用`spring.main.banner-mode`属性，我们可以关闭 Spring 横幅。

`spring.datasource.platform`设置数据库的供应商名称。 在初始化脚本中使用它。 `spring.jpa.show-sql`允许记录 SQL 语句。 最后，`spring.jpa.hibernate.ddl-auto`禁止从实体自动创建模式。

`com/zetcode/model/City.java`

```java
package com.zetcode.model;

import java.util.Objects;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

@Entity
@Table(name = "cities")
public class City {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    private String name;
    private int population;

    public City() {
    }

    public City(String name, int population) {
        this.name = name;
        this.population = population;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getPopulation() {
        return population;
    }

    public void setPopulation(int population) {
        this.population = population;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 79 * hash + Objects.hashCode(this.id);
        hash = 79 * hash + Objects.hashCode(this.name);
        hash = 79 * hash + this.population;
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final City other = (City) obj;
        if (this.population != other.population) {
            return false;
        }
        if (!Objects.equals(this.name, other.name)) {
            return false;
        }
        return Objects.equals(this.id, other.id);
    }

    @Override
    public String toString() {

        var builder = new StringBuilder();
        builder.append("City{id=").append(id).append(", name=")
                .append(name).append(", population=")
                .append(population).append("}");

        return builder.toString();
    }
}

```

这是`City`实体。 每个实体必须至少定义两个注解：`@Entity`和`@Id`。

```java
@Entity
@Table(name = "cities")
public class City {

```

`@Entity`注解指定该类是一个实体，并映射到数据库表。 `@Table`注解指定要用于映射的数据库表的名称。

```java
@Id
@GeneratedValue(strategy = GenerationType.AUTO)
private Long id;

```

`@Id`注解指定实体的主键，`@GeneratedValue`提供规范主键值的生成策略。

`resources/schema-h2.sql`

```java
CREATE TABLE cities(id INT PRIMARY KEY AUTO_INCREMENT,
    name VARCHAR(255), population INT);

```

启动应用时，将执行`schema-h2.sql`脚本。 它创建一个新的数据库表。

`resources/data-h2.sql`

```java
INSERT INTO cities(name, population) VALUES('Bratislava', 432000);
INSERT INTO cities(name, population) VALUES('Budapest', 1759000);
INSERT INTO cities(name, population) VALUES('Prague', 1280000);
INSERT INTO cities(name, population) VALUES('Warsaw', 1748000);
INSERT INTO cities(name, population) VALUES('Los Angeles', 3971000);
INSERT INTO cities(name, population) VALUES('New York', 8550000);
INSERT INTO cities(name, population) VALUES('Edinburgh', 464000);
INSERT INTO cities(name, population) VALUES('Suzhou', 4327066);
INSERT INTO cities(name, population) VALUES('Zhengzhou', 4122087);
INSERT INTO cities(name, population) VALUES('Berlin', 3671000);

```

之后，执行`data-h2.sql`文件。 它用数据填充表。

`com/zetcode/repository/CityRepository.java`

```java
package com.zetcode.repository;

import com.zetcode.model.City;
import java.util.List;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;
import org.springframework.stereotype.Repository;

@Repository
public interface CityRepository extends CrudRepository<City, Long> {

    @Query("select c from City c where c.name like %?1")
    List<City> findByNameEndsWith(String chars);
}

```

通过从 Spring `CrudRepository`扩展，我们将为数据存储库实现一些方法。 另外，我们创建一个自定义的`findByNameEndsWith()`方法。

```java
@Query("select c from City c where c.name like %?1")
List<City> findByNameEndsWith(String chars);

```

`@Query`注解包含自定义 JPQL 查询。 它返回名称以提供的字符结尾的城市。

`com/zetcode/service/ICityService.java`

```java
package com.zetcode.service;

import com.zetcode.model.City;
import java.util.List;

public interface ICityService {

    List<City> findByNameEndsWith(String name);
}

```

`ICityService`提供了一种获取所有以特定字符结尾的城市的契约方法。

`com/zetcode/service/CityService.java`

```java
package com.zetcode.service;

import com.zetcode.model.City;
import com.zetcode.repository.CityRepository;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CityService implements ICityService {

    @Autowired
    private CityRepository repository;

    @Override
    public List<City> findByNameEndsWith(String name) {

        var cities = (List<City>) repository.findByNameEndsWith(name);
        return cities;
    }
}

```

`CityService`包含`findByNameEndsWith()`方法的实现。 我们使用存储库从数据库检索数据。

```java
@Autowired
private CityRepository repository;

```

注入`CityRepository`。

```java
var cities = (List<City>) repository.findByNameEndsWith(name);

```

存储库的`findByNameEndsWith()`方法返回以某些字符结尾的城市列表。

`com/zetcode/controller/MyController.java`

```java
package com.zetcode.controller;

import com.zetcode.model.City;
import com.zetcode.service.ICityService;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

@Controller
public class MyController {

    @Autowired
    ICityService cityService;

    @GetMapping("/showCitiesEnding")
    public String findCitiesNameEndsWith(Model model, @RequestParam String name) {

        var cities = (List<City>) cityService.findByNameEndsWith(name);

        model.addAttribute("cities", cities);

        return "showCities";
    }
}

```

`MyController`类用`@Controller`注解。

```java
@Autowired
private ICityService cityService;

```

我们在`cityService`字段中插入`ICityService`。

```java
@GetMapping("/showCitiesEnding")
public String findCitiesNameEndsWith(Model model, @RequestParam String name) {

    var cities = (List<City>) cityService.findByNameEndsWith(name);

    model.addAttribute("cities", cities);

    return "showCities";
}

```

我们将带有`showCitiesEnding`路径的请求映射到控制器的`findCitiesNameEndsWith()`方法。 该模型将获取匹配城市的列表，并将处理过程发送到`showCities.ftl` Freemarker 模板文件。

`resources/templates/showCities.ftl`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Cities</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        </head>
    <body>
        <h2>List of cities</h2>

        <table>
            <tr>
                <th>Id</th>
                <th>Name</th>
                <th>Population</th>
            </tr>

            <#list cities as city>
                <tr>
                    <td>${city.id}</td>
                    <td>${city.name}</td>
                    <td>${city.population}</td>
                </tr>
            </#list>
        </table>
    </body>
</html>

```

在`showCities.ftl`模板文件中，我们在 HTML 表中显示数据。

`resources/static/index.html`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Home page</title>
        <meta charset="UTF-8"/>
        <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
    </head>
    <body>
        <a href="showCitiesEnding?name=ou">Show cities ending in ou</a>
    </body>
</html>

```

在`index.html`中有一个链接，显示以`ou`字符结尾的城市。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`设置 Spring Boot 应用。 `@SpringBootApplication`启用自动配置和组件扫描。

```java
$ mvn spring-boot:run

```

应用运行后，我们可以导航到`localhost:8080/myapp/`。

在本教程中，我们展示了如何使用 Spring Data JPA `@Query`注解创建自定义 JPQL 查询。 您可能也对相关教程感兴趣：

*   [Spring Boot MySQL 教程](/springboot/mysql/)
*   [Spring Boot Data JPA `@NamedQuery`教程](/springboot/datajpanamedquery/)
*   [Spring Boot REST Data JPA 教程](/articles/springbootrestdatajpa/)
[](/articles/springbootrestdatajpa/)
*   [Spring Boot Data JPA 排序教程](/springboot/datajpasort/)
*   [Spring Boot `CrudRepository`教程](/springboot/crudrepository/)
*   [Spring Boot H2 REST 教程](/articles/springbootresth2/)

列出[所有 Spring Boot 教程](/all/#springboot)。