# Spring Boot `@DataJpaTest`教程

> 原文： [http://zetcode.com/springboot/datajpatest/](http://zetcode.com/springboot/datajpatest/)

Spring Boot `@DataJpaTest`教程显示了如何使用`@DataJpaTest`注解测试 JPA 存储库。

Spring 是用于创建企业应用的流行 Java 应用框架。 Spring Boot 是 Spring 框架的演进，可帮助您轻松创建独立的，生产级的基于 Spring 的应用。

## `@DataJpaTest`

`@DataJpaTest`用于测试 JPA 信息库。 与`@RunWith(SpringRunner.class)`结合使用。 注解会禁用完全自动配置，并且仅应用与 JPA 测试相关的配置。 默认情况下，带有`@DataJpaTest`注解的测试使用嵌入式内存数据库。

在我们的测试中，我们可以从应用中注入`DataSource`，`@JdbcTemplate`，`@EntityManager`或任何 Spring Data 存储库。

包含所有这些组件（包括内存数据库）的应用上下文在所有用`@DataJpaTest`注解的测试类中的所有测试方法之间共享。 因此，每个测试方法都在自己的事务中运行，该事务在方法执行后会回滚。 这样，测试就彼此保持独立。

## Spring `@DataJpaTest`示例

下面的应用创建一个自定义的 JPA 查询方法。 该方法在带有`@DataJpaTest`注解的测试类中进行测试。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           │   MyRunner.java
│   │           ├───model
│   │           │       City.java
│   │           └───repository
│   │                   CityRepository.java
│   └───resources
│           application.properties
│           data-h2.sql
│           schema-h2.sql
└───test
    └───java
        └───com
            └───zetcode
                └───repository
                        CityRepositoryTest.java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springbootdatajpatest</artifactId>
    <version>1.0-SNAPSHOT</version>

    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-data-jpa</artifactId>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-test</artifactId>
            <scope>test</scope>
        </dependency>

        <dependency>
            <groupId>com.h2database</groupId>
            <artifactId>h2</artifactId>
            <scope>runtime</scope>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>

        </plugins>
    </build>

</project>

```

Maven POM 文件包含 Spring Data JPA，测试和 H2 数据库的依赖项。

`resources/application.properties`

```java
spring.main.banner-mode=off
spring.datasource.platform=h2
spring.jpa.hibernate.ddl-auto=none

```

`application.properties`是主要的 Spring Boot 配置文件。 使用`spring.main.banner-mode`属性，我们可以关闭 Spring 横幅。 `spring.datasource.platform`设置数据库的供应商名称。 在初始化脚本中使用它。 最后，`spring.jpa.hibernate.ddl-auto`禁止从实体自动创建模式。

`resources/schema-h2.sql`

```java
CREATE TABLE cities(id INT PRIMARY KEY AUTO_INCREMENT,
    name VARCHAR(255), population INT);

```

启动应用时，将执行`schema-h2.sql`脚本。 它创建一个新的数据库表。

`resources/data-h2.sql`

```java
INSERT INTO cities(name, population) VALUES('Bratislava', 432000);
INSERT INTO cities(name, population) VALUES('Budapest', 1759000);
INSERT INTO cities(name, population) VALUES('Prague', 1280000);
INSERT INTO cities(name, population) VALUES('Warsaw', 1748000);
INSERT INTO cities(name, population) VALUES('Los Angeles', 3971000);
INSERT INTO cities(name, population) VALUES('New York', 8550000);
INSERT INTO cities(name, population) VALUES('Edinburgh', 464000);
INSERT INTO cities(name, population) VALUES('Suzhou', 4327066);
INSERT INTO cities(name, population) VALUES('Zhengzhou', 4122087);
INSERT INTO cities(name, population) VALUES('Berlin', 3671000);
INSERT INTO cities(name, population) VALUES('Brest', 139163);
INSERT INTO cities(name, population) VALUES('Bucharest', 1836000);

```

之后，执行`data-h2.sql`文件。 它用数据填充表。

`com/zetcode/model/City.java`

```java
package com.zetcode.model;

import java.util.Objects;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

@Entity
@Table(name = "cities")
public class City {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    private String name;
    private int population;

    public City() {
    }

    public City(String name, int population) {
        this.name = name;
        this.population = population;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getPopulation() {
        return population;
    }

    public void setPopulation(int population) {
        this.population = population;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 79 * hash + Objects.hashCode(this.id);
        hash = 79 * hash + Objects.hashCode(this.name);
        hash = 79 * hash + this.population;
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final City other = (City) obj;
        if (this.population != other.population) {
            return false;
        }
        if (!Objects.equals(this.name, other.name)) {
            return false;
        }
        return Objects.equals(this.id, other.id);
    }

    @Override
    public String toString() {

        var builder = new StringBuilder();
        builder.append("City{id=").append(id).append(", name=")
                .append(name).append(", population=")
                .append(population).append("}");

        return builder.toString();
    }
}

```

这是`City`实体。

`com/zetcode/repository/CityRepository.java`

```java
package com.zetcode.repository;

import com.zetcode.model.City;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.CrudRepository;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;

import java.util.List;

@Repository
public interface CityRepository extends CrudRepository<City, Long> {

    @Query("SELECT c FROM City c WHERE c.name LIKE CONCAT('%',:ending, '%') AND c.population < :num")
    List<City> findByNameEndingWithAndPopulationLessThan(@Param("ending") String ending,
                                                         @Param("num") Integer num);
}

```

`CityRepository`包含自定义的`findByNameEndingWithAndPopulationLessThan()`方法。 使用该方法，我们可以得到所有以指定字符串结尾且其人口小于指定值的城市名称。

`com/zetcode/MyRunner.java`

```java
package com.zetcode;

import com.zetcode.repository.CityRepository;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

@Component
public class MyRunner implements CommandLineRunner {

    private static final Logger logger = LoggerFactory.getLogger(MyRunner.class);

    @Autowired
    private CityRepository cityRepository;

    @Override
    public void run(String... args) throws Exception {

        var cities = cityRepository.findByNameEndingWithAndPopulationLessThan("est", 1800000);
        cities.forEach(city -> logger.info("{}", city));
    }
}

```

在`MyRunner`中，我们使用`findByNameEndingWithAndPopulationLessThan()`方法。

> **注意**：在Java企业应用程序中，定义与存储库一起使用的服务层是一个好习惯。 为简单起见，我们跳过服务层。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`设置 Spring Boot 应用。 `@SpringBootApplication`启用自动配置和组件扫描。

`com/zetcode/repository/CityRepositoryTest.java`

```java
package com.zetcode.repository;

import com.zetcode.model.City;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.orm.jpa.DataJpaTest;
import org.springframework.test.context.junit4.SpringRunner;

import static org.assertj.core.api.Assertions.assertThat;

@RunWith(SpringRunner.class)
@DataJpaTest
public class CityRepositoryTest {

    @Autowired
    private CityRepository repository;

    @Test
    public void should_find_all_customers() {

        Iterable<City> cities = repository.findAll();

        int nOfCities = 12;
        assertThat(cities).hasSize(nOfCities);
    }

    @Test
    public void should_find_with_name_ending_population_less_than() {

        var cities = repository.findByNameEndingWithAndPopulationLessThan("est", 150000);

        assertThat(cities).isNotEmpty();
    }

}

```

在`CityRepositoryTest`中，我们测试了自定义 JPA 方法。

```java
@RunWith(SpringRunner.class)
@DataJpaTest
public class CityRepositoryTest {

```

`CityRepositoryTest`带有`@DataJpaTest`注解。 内存中的 H2 数据库用于执行集成测试。

```java
@Test
public void should_find_with_name_ending_population_less_than() {

    var cities = repository.findByNameEndingWithAndPopulationLessThan("est", 150000);

    assertThat(cities).isNotEmpty();
}

```

此方法测试至少有一个城市的名字以“`est`”结尾并且人口少于 150000。

```java
$ mvn spring-boot:test

```

我们运行测试。

在本教程中，我们展示了如何使用`@DataJpaTest`测试自定义 JPA 存储库方法。

*   [Spring Boot `TestEntityManager`教程](/springboot/testentitymanager/)
*   [Spring Boot Data JPA `@NamedQuery`教程](/springboot/datajpanamedquery/)
*   [Spring Boot REST Data JPA 教程](/articles/springbootrestdatajpa/)
*   [Spring Boot Data JPA 排序教程](/springboot/datajpasort/)
*   [Spring Boot `CrudRepository`教程](/springboot/crudrepository/)

列出[所有 Spring Boot 教程](/all/#springboot)。