# Spring Boot CSV 教程

> 原文： [http://zetcode.com/articles/springbootcsv/](http://zetcode.com/articles/springbootcsv/)

在本教程中，我们将在 Spring Boot RESTful 应用中以 CSV 格式提供内容。 我们使用 OpenCSV 库。

Spring 是用于创建企业应用的流行 Java 应用框架。 Spring Boot 是 Spring 框架发展的下一步。 它有助于以最小的努力创建独立的，基于生产级的 Spring 应用。 它放弃了 XML 配置，并大量使用约定而不是配置原则。

CSV（逗号分隔值）是一种简单的数据格式，其中（大多数）值之间用逗号分隔，每一行代表一个记录。 数据存储在纯文本文件中。 作为电子表格和数据库中使用的导入和导出格式，它非常流行。 OpenCSV 是 Java 的开源，简单 CSV 解析器库。

Hibernate 是 Java 语言的对象关系映射框架。 它提供了一个框架，用于将面向对象的域模型映射到关系数据库。 对象关系映射（ORM）是一种编程技术，用于在面向对象的编程语言中的不兼容类型系统之间转换数据。

Spring Data JPA 是 Spring Data 项目的一部分，该项目使实现基于 JPA 的存储库变得更加容易。 Spring Data JPA 使用 JPA 将数据存储在关系数据库中。 它可以在运行时从存储库界面自动创建存储库实现。

RESTFul 应用遵循 REST 体系结构样式，该样式用于设计网络应用。 RESTful 应用生成 HTTP 请求，这些请求对资源执行 CRUD（创建/读取/更新/删除）操作。

## Spring Boot CSV 示例

我们的应用是一个 Spring Boot RESTful 应用，它以 CSV 格式从 H2 数据库返回数据。

```java
$ tree
.
├── pom.xml
└── src
    ├── main
    │   ├── java
    │   │   └── com
    │   │       └── zetcode
    │   │           ├── Application.java
    │   │           ├── bean
    │   │           │   └── City.java
    │   │           ├── controller
    │   │           │   └── MyCsvController.java
    │   │           ├── repository
    │   │           │   └── CityRepository.java
    │   │           ├── service
    │   │           │   ├── CityService.java
    │   │           │   └── ICityService.java
    │   │           └── util
    │   │               └── WriteCsvToResponse.java
    │   └── resources
    │       ├── application.yml
    │       └── import.sql
    └── test
        └── java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0" 
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" 
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 
                             http://maven.apache.org/xsd/maven-4.0.0.xsd">

    <modelVersion>4.0.0</modelVersion>
    <groupId>com.zetcode</groupId>
    <artifactId>SpringBootCSV</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>
    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>1.8</maven.compiler.source>
        <maven.compiler.target>1.8</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>1.5.3.RELEASE</version>
    </parent>      

    <dependencies>
        <dependency>
            <groupId>com.h2database</groupId>
            <artifactId>h2</artifactId>
            <scope>runtime</scope>
        </dependency>

        <dependency>
            <groupId>com.opencsv</groupId>
            <artifactId>opencsv</artifactId>
            <version>3.9</version>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-web</artifactId>
        </dependency>      

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-data-jpa</artifactId>
        </dependency>        

    </dependencies>    

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>            
        </plugins>
    </build>      
</project>

```

这是 Maven 构建文件。 `h2`依赖项添加了一个 H2 数据库驱动程序。 `opencsv`依赖项添加了 OpenCSV 库的驱动程序。

Spring Boot 启动器是一组方便的依赖项描述符，我们可以在我们的应用中包含这些描述符。 它们极大地简化了 Maven 配置。 `spring-boot-starter-parent`为 Spring Boot 应用提供了一些常用配置。 `spring-boot-starter-web`是使用 Spring MVC 构建 Web（包括 RESTful）应用的入门工具。 它使用 Tomcat 作为默认的嵌入式容器。 `spring-boot-starter-data-jpa`是将 Spring Data JPA 与 Hibernate 结合使用的入门工具。

在`spring-boot-maven-plugin`提供了 Maven 的 Spring Boot 支持，使我们能够打包可执行的 JAR 或 WAR 档案。 它的`spring-boot:run`目标运行 Spring Boot 应用。

`application.yml`

```java
server:
    port: 8086
    context-path: /rest

spring: 
    main:
        banner-mode: "off"     
    jpa:
        database: h2
        hibernate:
            dialect: org.hibernate.dialect.H2Dialect
            ddl-auto: create-drop

logging: 
    level: 
        org: 
            springframework: ERROR

```

`application.yml`文件包含 Spring Boot 应用的各种配置设置。 我们具有服务器端口和上下文路径（应用名称）的映射。 我们通过`localhost:8086/rest/` URL 访问我们的应用。 使用`banner-mode`属性，我们可以关闭 Spring 横幅。

JPA `database`值指定要操作的目标数据库。 在本例中，我们指定了 Hibernate 方言`org.hibernate.dialect.H2Dialect`。 `ddl-auto`是数据定义语言模式； `create-drop`选项将自动创建和删除数据库模式。

H2 数据库在内存中运行。 另外，我们将 spring 框架的日志记录级别设置为`ERROR`。 在`application.yml`文件位于中`src/main/resources`目录。

`City.java`

```java
package com.zetcode.bean;

import java.io.Serializable;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

@Entity
@Table(name = "CITIES")
public class City implements Serializable {

    private static final long serialVersionUID = 1L;

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    private String name;
    private int population;

    public City() {
    }

    public City(Long id, String name, int population) {
        this.id = id;
        this.name = name;
        this.population = population;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getPopulation() {
        return population;
    }

    public void setPopulation(int population) {
        this.population = population;
    }

    @Override
    public String toString() {
        return "City{" + "id=" + id + ", name=" + name
                + ", population=" + population + '}';
    }
}

```

这是`City`实体。 每个实体必须至少定义两件事：`@Entity`注解和带有`@Id`注解的 ID 字段。 我们已经将`ddl-auto`选项设置为`create-drop`，这意味着 Hibernate 将根据该实体创建表模式。

```java
@Entity
@Table(name = "CITIES")
public class City implements Serializable {

```

`@Entity`注解指定该类是一个实体，并映射到数据库表。 `@Table`实体指定要用于映射的数据库表的名称。

```java
@Id
@GeneratedValue(strategy = GenerationType.AUTO)
private Long id;

```

`@Id`注解指定实体的主键，`@GeneratedValue`提供规范主键值的生成策略。

`import.sql`

```java
INSERT INTO CITIES(NAME, POPULATION) VALUES('Bratislava', 432000);
INSERT INTO CITIES(NAME, POPULATION) VALUES('Budapest', 1759000);
INSERT INTO CITIES(NAME, POPULATION) VALUES('Prague', 1280000);
INSERT INTO CITIES(NAME, POPULATION) VALUES('Warsaw', 1748000);
INSERT INTO CITIES(NAME, POPULATION) VALUES('Los Angeles', 3971000);
INSERT INTO CITIES(NAME, POPULATION) VALUES('New York', 8550000);
INSERT INTO CITIES(NAME, POPULATION) VALUES('Edinburgh', 464000);
INSERT INTO CITIES(NAME, POPULATION) VALUES('Berlin', 3671000);

```

模式是由 Hibernate 自动创建的。 之后，将执行`import.sql`文件以将数据填充到表中。

`CityRepository.java`

```java
package com.zetcode.repository;

import com.zetcode.bean.City;
import org.springframework.data.repository.CrudRepository;
import org.springframework.stereotype.Repository;

@Repository
public interface CityRepository extends CrudRepository<City, Long> {

}

```

通过从 Spring `CrudRepository`扩展，我们将为我们的数据存储库实现一些方法，包括`findAll()`和`findOne()`。 这样，我们节省了大量样板代码。

`ICityService.java`

```java
package com.zetcode.service;

import com.zetcode.bean.City;
import java.util.List;

public interface ICityService {

    public List<City> findAll();
    public City findById(Long id);
}

```

`ICityService`提供了获取所有城市并通过其 ID 从数据源获取城市的契约方法。

`CityService.java`

```java
package com.zetcode.service;

import com.zetcode.bean.City;
import com.zetcode.repository.CityRepository;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CityService implements ICityService {

    @Autowired
    private CityRepository repository;

    @Override
    public List<City> findAll() {

        List<City> cities = (List<City>) repository.findAll();

        return cities;
    }

    @Override
    public City findById(Long id) {

        City city = repository.findOne(id);
        return city;
    }
}

```

`CityService`包含`findAll()`和`findById()`方法的实现。 我们使用存储库从数据库检索数据。

```java
@Autowired
private CityRepository repository;

```

注入`CityRepository`。

```java
List<City> cities = (List<City>) repository.findAll();

```

存储库的`findAll()`方法返回城市列表。

```java
City city = repository.findOne(id);

```

存储库的`findOne()`方法返回一个特定的城市对象。

`MyCsvController.java`

```java
package com.zetcode.controller;

import com.zetcode.bean.City;
import com.zetcode.service.ICityService;
import com.zetcode.util.WriteCsvToResponse;
import java.io.IOException;
import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class MyCsvController {

    @Autowired
    ICityService cityService;

    @RequestMapping(value = "/cities", produces = "text/csv")
    public void findCities(HttpServletResponse response) throws IOException {

        List<City> cities = (List<City>) cityService.findAll();

        WriteCsvToResponse.writeCities(response.getWriter(), cities);
    }

    @RequestMapping(value = "/cities/{cityId}", produces = "text/csv")
    public void findCity(@PathVariable Long cityId, HttpServletResponse response) throws IOException {

        City city = cityService.findById(cityId);
        WriteCsvToResponse.writeCity(response.getWriter(), city);
    }
}

```

这是 Spring Boot RESTful 应用的控制器类。 `@RestController`注解创建一个 RESTful 控制器。 传统的 MVC 控制器使用`ModelAndView`，而 RESTful 控制器仅返回对象，并且对象数据通常以 JSON 或 XML 格式直接写入 HTTP 响应（通常）。 在本例中，我们选择了 CSV 格式。

```java
@Autowired
private ICityService cityService;

```

我们在`countryService`字段中插入`ICityService`。

```java
@RequestMapping(value = "/cities", produces = "text/csv")
public void findCities(HttpServletResponse response) throws IOException {

...
}

```

`@RequestMapping`注解用于将 Web 请求映射到 Spring 控制器方法。 `produces`选项设置媒体类型，在本例中为`text/csv`。 我们将带有`/cities`路径的请求映射到控制器的`findCities()`方法。 默认请求是 GET 请求。

```java
List<City> cities = (List<City>) cityService.findAll();

WriteCsvToResponse.writeCities(response.getWriter(), cities);

```

我们称`cityService`的`findAll()`来获取所有城市。 我们将 CSV 数据写入`HttpServletResponse`对象。 Java Bean 与 CSV 数据的映射委托给`WriteCsvToResponse`类。

```java
@RequestMapping(value = "/cities/{cityId}", produces = "text/csv")
public void findCity(@PathVariable Long cityId, HttpServletResponse response) throws IOException {

    City city = cityService.findById(cityId);
    WriteCsvToResponse.writeCity(response.getWriter(), city);
}

```

在第二种方法中，我们有一个 URL 路径，其中包含要检索的城市的 ID。 我们使用`@PathVariable`注解将 URL 模板变量绑定到方法`cityId`参数。

`WriteCsvToResponse.java`

```java
package com.zetcode.util;

import com.opencsv.CSVWriter;
import com.opencsv.bean.ColumnPositionMappingStrategy;
import com.opencsv.bean.StatefulBeanToCsv;
import com.opencsv.bean.StatefulBeanToCsvBuilder;
import com.opencsv.exceptions.CsvException;
import com.zetcode.bean.City;
import java.io.PrintWriter;
import java.util.List;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class WriteCsvToResponse {

    private static final Logger LOGGER = LoggerFactory.getLogger(WriteCsvToResponse.class);

    public static void writeCities(PrintWriter writer, List<City> cities)  {

        try {

            ColumnPositionMappingStrategy mapStrategy
                    = new ColumnPositionMappingStrategy();

            mapStrategy.setType(City.class);
            mapStrategy.generateHeader();

            String[] columns = new String[]{"id", "name", "population"};
            mapStrategy.setColumnMapping(columns);

            StatefulBeanToCsv btcsv = new StatefulBeanToCsvBuilder(writer)
                    .withQuotechar(CSVWriter.NO_QUOTE_CHARACTER)
                    .withMappingStrategy(mapStrategy)
                    .withSeparator(',')
                    .build();

            btcsv.write(cities);

        } catch (CsvException ex) {

            LOGGER.error("Error mapping Bean to CSV", ex);
        }
    }

    public static void writeCity(PrintWriter writer, City city) {

        try {

            ColumnPositionMappingStrategy mapStrategy
                    = new ColumnPositionMappingStrategy();

            mapStrategy.setType(City.class);

            String[] columns = new String[]{"id", "name", "population"};
            mapStrategy.setColumnMapping(columns);

            StatefulBeanToCsv btcsv = new StatefulBeanToCsvBuilder(writer)
                    .withQuotechar(CSVWriter.NO_QUOTE_CHARACTER)
                    .withMappingStrategy(mapStrategy)
                    .withSeparator(',')
                    .build();

            btcsv.write(city);

        } catch (CsvException ex) {

            LOGGER.error("Error mapping Bean to CSV", ex);
        }
    }
}

```

在`WriteCsvToResponse`中，我们使用 OpenCSV 库将 Java bean 转换为 CSV，并将最终输出写入`HttpServletResponse`中。

```java
ColumnPositionMappingStrategy mapStrategy
        = new ColumnPositionMappingStrategy();

mapStrategy.setType(City.class);

```

`MappingStrategy`定义如何将 Java 属性映射到 CSV 列名称。 `ColumnPositionMappingStrategy`使用列位置进行映射。

```java
String[] columns = new String[]{"id", "name", "population"};
mapStrategy.setColumnMapping(columns);

```

我们设置列名。

```java
StatefulBeanToCsv btcsv = new StatefulBeanToCsvBuilder(writer)
        .withQuotechar(CSVWriter.NO_QUOTE_CHARACTER)
        .withMappingStrategy(mapStrategy)
        .withSeparator(',')
        .build();

```

`StatefulBeanToCsv`类以 CSV 格式写出 bean 给写程序，以保留状态信息并明智地猜测要应用的映射策略。

```java
btcsv.write(city);

```

最后，编写了 bean。

`Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`设置 Spring Boot 应用。 `@SpringBootApplication`启用自动配置和组件扫描。

```java
$ mvn spring-boot:run

```

使用`mvn spring-boot:run`命令，运行应用。 该应用部署在嵌入式 Tomcat 服务器上。

```java
$ curl localhost:8086/rest/cities
1,Bratislava,432000
2,Budapest,1759000
3,Prague,1280000
4,Warsaw,1748000
5,Los Angeles,3971000
6,New York,8550000
7,Edinburgh,464000
8,Berlin,3671000

```

使用`curl`命令，我们可以获得所有城市。

```java
$ curl localhost:8086/rest/cities/1
1,Bratislava,432000

```

在这里，我们得到了一个由其 ID 标识的城市。

在本教程中，我们已从 Spring Boot RESTful 应用以 CSV 格式将数据返回给客户端。 我们使用了 OpenCSV 库。 您可能也对相关教程感兴趣：

*   [在 Spring Boot](/articles/springbootloadres/) 中加载资源
*   [Spring Boot H2 REST 教程](/articles/springbootresth2/)
*   [Spring Boot Thymeleaf 教程](/articles/springbootthymeleaf/)
*   [Spring Boot Mustache 教程](/articles/springbootmustache/)
*   [Spring Boot Swing 集成教程](/articles/springbootswing/)
*   [Spring Web 应用介绍](/articles/springwebfirst/)
*   [Spring Boot RESTFul 应用](/articles/springbootrestsimple/)
*   [经典 Spring 应用中的`JdbcTemplate`](/articles/springjdbctemplate/)