# Spring Boot `WebFlux` 教程

> 原文： [http://zetcode.com/springboot/webflux/](http://zetcode.com/springboot/webflux/)

Spring Boot WebFlux 教程展示了如何使用 WebFlux 创建一个简单的 Spring Boot 反应式 Web 应用。

## WebFlux

WebFlux 是一个 Spring Web 反应式框架。 它已添加到 Spring 5 中。它是完全非阻塞的，支持反应流背压，并在 Netty，Undertow 和 Servlet 3.1+容器等服务器上运行。

Spring WebFlux 是传统 Spring MVC 的替代方法。

Spring WebFlux 在内部使用 Project Reactor 及其发布者实现 Flux 和 Mono。 它支持两种编程模型：a）基于注解的反应组件，b）功能路由和处理。

## 反应式编程

反应式编程是一种编程范例，它是功能性的，基于事件的，非阻塞的，异步的，并且以数据流处理为中心。 术语反应式来自以下事实：我们对诸如鼠标单击或 I/O 事件之类的更改做出反应。

当我们处理大量流数据时，响应式应用可以更好地扩展，并且效率更高。 反应性应用是非阻塞的； 他们没有使用资源等待流程完成。

响应式应用实现基于事件的模型，在该模型中将数据推送到使用者。 数据的使用者（订阅者）订阅发布者，后者发布异步数据流。

## Spring Reactor

Spring Reactor 是一个反应式库，用于根据反应式流规范在 JVM 上构建非阻塞应用。

Reactor 项目提供两种类型的发布者：`Mono`和`Flux`。 `Flux`是产生 0 到 N 个值的发布者。 返回多个元素的操作使用此类型。 `Mono`是产生 0 到 1 值的发布者。 它用于返回单个元素的操作。

## Spring Boot `WebFlux` 示例

在以下应用中，我们创建一个带有响应支持的简单 Spring Boot Web 应用。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           └───controller
│   │                   MyController.java
│   └───resources
│           application.properties
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springbootwebfluxsimple</artifactId>
    <version>1.0-SNAPSHOT</version>

    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.5.RELEASE</version>
    </parent>

    <dependencies>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-webflux</artifactId>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>

        </plugins>
    </build>
</project>

```

这是 Maven `pom.xml`文件。 `spring-boot-starter-webflux`是用于使用 Spring Framework 的反应式 Web 支持构建 WebFlux 应用的 Spring Boot 入门程序。

`resources/application.properties`

```java
spring.main.banner-mode=off

```

在`application.properties`中，我们关闭 Spring Boot 横幅。

`com/zetcode/MyController.java`

```java
package com.zetcode.controller;

import org.reactivestreams.Publisher;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;
import reactor.core.publisher.Mono;

@RestController
public class MyController {

    @GetMapping("/")
    public Publisher<String> home() {

        return Mono.just("Home page");
    }
}

```

我们有一个简单的 REST 端点，它返回一条消息。 `home()`方法的返回类型是`Publisher`。 `Mono.just()`发出指定的字符串消息。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application  {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

这段代码设置了 Spring Boot 应用。

```java
$ mvn spring-boot:run

```

我们运行该应用并导航到`localhost:8080`。

在本教程中，我们创建了一个简单的 Spring Boot WebFlux 应用。

列出[所有 Spring Boot](/all/#springboot) 教程。