# Spring Boot `@ResponseBody`教程

> 原文： [http://zetcode.com/springboot/responsebody/](http://zetcode.com/springboot/responsebody/)

在 Spring Boot `@ResponseBody`教程中，我们将在控制器中使用 Spring `@ResponseBody`注解将数据写入响应对象的主体。

Spring 是流行的 Java 应用框架，而 Spring Boot 是 Spring 的演进，可以帮助轻松地创建独立的，生产级的基于 Spring 的应用。

WebJars 是打包到 JAR 文件中的客户端 Web 库（例如 jQuery 或 Bootstrap）。 它们允许轻松管理 Java Web 应用中的客户端依赖性

JQuery 是一个流行的开源 JavaScript 库，旨在简化 HTML 的客户端脚本。

## Spring `@ResponseBody`

`@ResponseBody`是一个 Spring 注解，它将方法返回值绑定到 Web 响应主体。 它不解释为视图名称。 它使用 HTTP 消息转换器根据请求 HTTP 标头中的内容类型将返回值转换为 HTTP 响应主体。

## Spring `@ResponseBody`示例

以下示例创建一个 Spring Boot Web 应用，该应用将 JSON 数据返回到客户端。 主页是通过 MVC 机制处理的； FreeMarker 用于创建主页的模板。 主页包含一个按钮，该按钮发送获取 JSON 数据的请求。 Spring Boot Web 应用在`@ResponseBody`注解的帮助下以 JSON 格式发送数据。

```java
pom.xml
src
├── main
│   ├── java
│   │   └── com
│   │       └── zetcode
│   │           ├── Application.java
│   │           ├── model
│   │           │   └── City.java
│   │           ├── controller
│   │           │   └── MyController.java
│   │           └── service
│   │               ├── CityService.java
│   │               └── ICityService.java
│   └── resources
│       └── templates
│           └── index.ftl
└── test
    └── java

```

这是应用的项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
http://maven.apache.org/xsd/maven-4.0.0.xsd">

    <modelVersion>4.0.0</modelVersion>
    <groupId>com.zetcode</groupId>
    <artifactId>springbootresponsebodyex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>
        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-web</artifactId>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-freemarker</artifactId>
        </dependency>

        <dependency>
            <groupId>org.webjars</groupId>
            <artifactId>jquery</artifactId>
            <version>3.3.1</version>
        </dependency>

        <dependency>
            <groupId>org.webjars</groupId>
            <artifactId>webjars-locator</artifactId>
            <version>0.34</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>
</project>

```

在`spring-boot-starter-freemarker`是建立使用 FreeMarker 的观点 Spring MVC 的 Web 应用的启动。 我们将 Webjar 用于 JQuery。 `webjars-locator`自动解析任何 WebJars 资产的版本。 该应用打包到 JAR 文件中，并使用 Tomcat 作为嵌入式 Web 服务器。

`com/zetcode/model/City.java`

```java
package com.zetcode.model;

import java.util.Objects;

public class City {

    private Long id;
    private String name;
    private int population;

    public City() {
    }

    public City(Long id, String name, int population) {
        this.id = id;
        this.name = name;
        this.population = population;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getPopulation() {
        return population;
    }

    public void setPopulation(int population) {
        this.population = population;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        City city = (City) o;
        return population == city.population &&
                Objects.equals(id, city.id) &&
                Objects.equals(name, city.name);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, name, population);
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("City{");
        sb.append("id=").append(id);
        sb.append(", name='").append(name).append('\'');
        sb.append(", population=").append(population);
        sb.append('}');
        return sb.toString();
    }
}

```

这是`City` bean。 它具有`id`，`name`和`population`属性。

`com/zetcode/service/ICityService.java`

```java
package com.zetcode.service;

import com.zetcode.model.City;
import java.util.List;

public interface ICityService {

    List<City> findAll();
}

```

`ICityService`包含获取所有城市的契约方法。

`com/zetcode/service/CityService.java`

```java
package com.zetcode.service;

import com.zetcode.model.City;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class CityService implements ICityService {

    @Override
    public List<City> findAll() {

        var cities = new ArrayList<City>();

        cities.add(new City(1L, "Bratislava", 432000));
        cities.add(new City(2L, "Budapest", 1759000));
        cities.add(new City(3L, "Prague", 1280000));
        cities.add(new City(4L, "Warsaw", 1748000));
        cities.add(new City(5L, "Los Angeles", 3971000));
        cities.add(new City(6L, "New York", 8550000));
        cities.add(new City(7L, "Edinburgh", 464000));
        cities.add(new City(8L, "Berlin", 3671000));

        return cities;
    }
}

```

`CityService`返回八个城市对象。

`com/zetcode/controller/MyController.java`

```java
package com.zetcode.controller;

import com.zetcode.bean.City;
import com.zetcode.service.ICityService;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

@Controller
public class MyController {

    @Autowired
    ICityService cityService;

    @RequestMapping(path = "/")
    public String index() {

        return "index";
    }

    @RequestMapping(path = "/GetCities", produces = "application/json; charset=UTF-8")
    @ResponseBody
    public List<City> findCities() {

        var cities = (List<City>) cityService.findAll();

        return cities;
    }
}

```

控制器有两种方法。 `index()`方法返回主页视图。 `findCities()`方法返回城市列表作为 JSON 数据。

```java
@Controller
public class MyController {

```

`@Controller`注解表示我们有一个控制器类。

```java
@RequestMapping(path = "/")
public String index() {

    return "index";
}

```

`index()`方法返回`index`字符串，该字符串被解析为`index.ftl`视图。 该视图位于`src/main/resources/templates`目录中。 当 Spring 在 POM 文件中找到`spring-boot-starter-freemarker`工件时，它将自动配置 FreeMarker。

```java
@RequestMapping(path = "/GetCities", produces = "application/json; charset=UTF-8")
@ResponseBody
public List<City> findCities() {

    var cities = (List<City>) cityService.findAll();

    return cities;
}

```

对于`GetCities`路径，将调用`findCities()`方法。 `produces`参数指示该方法返回 JSON 数据； Spring `RequestResponseBodyMethodProcessor`通过使用`HttpMessageConverter`写入响应主体来处理用`@ResponseBody`注解的方法的返回值。 在我们的例子中，消息转换器是`MappingJackson2HttpMessageConverter`，它使用 Jackson 的`ObjectMapper`读取和写入 JSON。 （Jackson 是一个流行的 Java JSON 库。）

`resources/templates/index.ftl`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Home Page</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <script src="webjars/jquery/jquery.min.js"></script>
    </head>
<body>

    <button id="mybtn">Get cities</button>

    <div>
        <ul id="output">

        </ul>
    </div>

    <script>
        $('#mybtn').click(function () {

            $.getJSON('GetCities', function (data) {

                $("ul#output > li").remove();

                $.each(data, function (key, value) {
                    $("#output").append('<li>' + value['name'] + " " + value['population'] + '</li>');
                });
            });
        });
    </script>
</body>
</html>

```

`index.ftl`文件是主页的模板。 它包含一个按钮，该按钮对 Web 应用执行异步请求。 它将加载城市列表并将其写入 HTML 列表。

```java
<script src="webjars/jquery/jquery.min.js"></script>

```

我们包括 JQuery 库。 多亏`webjars-locator`，我们可以包括一个版本无关的 JQuery 库。 因此，如果 JQuery 的版本发生更改，我们不必更新链接。

```java
<script>
    $('#mybtn').click(function () {

        $.getJSON('GetCities', function (data) {

            $("ul#output > li").remove();

            $.each(data, function (key, value) {
                $("#output").append('<li>' + value['name'] + " " + value['population'] + '</li>');
            });
        });
    });
</script>

```

通过`$.getJSON()`方法，我们使用 HTTP GET 请求以 JSON 格式加载数据。 数据用`$.each()`遍历并写入 HTML 列表。

在本教程中，我们在 Spring Boot Web 应用中使用了`@ResponseBody`注解。 您可能也对相关教程感兴趣： [Spring Boot `ResponseEntity`教程](/springboot/responseentity/)， [Spring Boot `@PathVariable`教程](/springboot/pathvariable/)， [Spring Boot 上传文件](/springboot/uploadfile/)， [Spring Boot `@RequestParam`教程](/springboot/requestparam/)，[独立的 Spring 应用](/articles/standalonespring/)， [Java 教程](/lang/java/)或列出[所有 Spring Boot 教程](/all/#springboot)。