# Spring Boot `@PathVariable`教程

> 原文： [http://zetcode.com/springboot/pathvariable/](http://zetcode.com/springboot/pathvariable/)

Spring Boot `@PathVariable`教程展示了如何读取带有`@PathVariable`注解的 URL 模板变量。 我们创建一个 Spring Boot RESTful 应用来演示注解。

Spring 是流行的 Java 应用框架，而 Spring Boot 是 Spring 的演进，可以帮助轻松地创建独立的，生产级的基于 Spring 的应用。

## `@PathVariable`

`@PathVariable`是一个 Spring 注解，它指示方法参数应绑定到 URI 模板变量。

它具有以下可选元素：

*   `name` - 要绑定到的路径变量的名称
*   `required` - 指示路径变量是否为必需
*   `value` - 名称的别名

## Spring Boot `@PathVariable`示例

以下示例创建一个使用`@PathVariable`的 Spring Boot Web 应用。 该应用接收一个 URL，从该 URL 构建到客户端的文本响应。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           └───controller
│   │                   MyController.java
│   └───resources
└───test
    └───java

```

这是 Spring Boot 应用的项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>pathvariableex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-devtools</artifactId>
            <optional>true</optional>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-web</artifactId>
        </dependency>

    </dependencies>

</project>

```

这是 Maven 构建文件。 `spring-boot-starter-web`是使用 Spring MVC 构建 Web 应用的入门工具。 它使用 Tomcat 作为默认的嵌入式容器。 `spring-boot-devtools`是在开发 Spring Boot 应用时有用的构件。 它允许自动重启或实时重新加载应用。 该应用打包到一个 JAR 文件中。

`com/zetcode/controller/MyController.java`

```java
package com.zetcode.controller;

import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class MyController {

    @RequestMapping(path="/{name}/{age}")
    public String getMessage(@PathVariable("name") String name, 
            @PathVariable("age") String age) {

        var msg = String.format("%s is %s years old", name, age);

        return msg;
    }
}

```

控制器处理来自客户端的请求。 它从请求的 URL 中读取两个值。

```java
@RestController
public class MyController {

```

我们有一个 RESTful Web 应用。

```java
@RequestMapping(path="/{name}/{age}")
public String getMessage(@PathVariable("name") String name, 
        @PathVariable("age") String age) {

```

使用`@PathVariable`注解，我们将请求 URL 模板路径变量绑定到方法变量。 例如，对于`/July/28/` URL，七月值绑定到`name`变量，而 28 值绑定到`age`变量。

```java
var msg = String.format("%s is %s years old", name, age);

return msg;

```

我们构建消息并返回。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application  {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`是设置 Spring Boot 应用的入口。 `@SpringBootApplication`注解启用自动配置和组件扫描。

```java
$ mvn spring-boot:run

```

我们启动 Spring Boot 应用。

```java
$ curl localhost:8080/Robert/39/
Robert is 39 years old

```

我们使用`curl`工具向应用创建请求。 该应用响应一条消息。 使用`@PathVariable`从 URL 中提取值。

在本教程中，我们使用 Spring Boot 框架创建了一个 RESTful Web 应用。 我们已经演示了`@PathVariable`的用法。 您可能也对相关教程感兴趣： [Spring Boot `@RequestParam`教程](/springboot/requestparam/)， [Spring Boot `@ResponseBody`教程](/springboot/responsebody/)， [Spring Boot `@RestController`教程](/springboot/restcontrollre/)， [Java 教程](/lang/java/)，或列出[所有 Spring Boot 教程](/all/#springboot)。