# Spring Boot `@RestController`

> 原文： [http://zetcode.com/springboot/restcontroller/](http://zetcode.com/springboot/restcontroller/)

Spring Boot `@RestController`教程展示了如何在 Spring 应用中使用`@RestController`注解来构建 Restful 控制器。

Spring 是流行的 Java 应用框架，而 Spring Boot 是 Spring 的演进，可以帮助轻松地创建独立的，生产级的基于 Spring 的应用。

## Spring MVC

Spring MVC 是基于 Servlet API 构建的主要 Web 框架。 它基于流行的 MVC 设计模式。 MVC（模型-视图-控制器）是一种软件体系结构模式，它将应用分为三个区域：模型，视图和控制器。 该模型表示一个携带数据的 Java 对象。 该视图表示模型包含的数据的可视化。 控制器控制数据流到模型对象中，并在数据更改时更新视图。 它将视图和模型分开。

Spring Framework 5.0 引入了一个名为 Spring WebFlux 的并行反应式栈 Web 框架。

## `@RestController`

`@RestController`是用于创建 Restful 控制器的便捷注解。 它是`@Component`的特化，可通过类路径扫描自动检测。 它添加了`@Controller`和`@ResponseBody`注解。 它将响应转换为 JSON 或 XML。 它不适用于视图技术，因此方法无法返回`ModelAndView`。 它通常与基于`@RequestMapping`注解的注解处理器方法结合使用。

`@Controller`注解与视图技术一起使用。

## RESTFul 应用

RESTFul 应用遵循 REST 体系结构样式，该样式用于设计网络应用。 RESTful 应用生成对资源执行 CRUD（创建/读取/更新/删除）操作的 HTTP 请求。 RESTFul 应用通常以 JSON 或 XML 格式返回数据。

## Spring Boot `@RestController`示例

在以下应用中，我们演示`@RestController`的用法。 该应用返回城市列表作为 JSON 数据。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           ├───controller
│   │           │       MyController.java
│   │           ├───model
│   │           │       City.java
│   │           └───service
│   │                   CityService.java
│   │                   ICityService.java
│   └───resources
│       └───static
│               index.html
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
http://maven.apache.org/xsd/maven-4.0.0.xsd">

    <modelVersion>4.0.0</modelVersion>
    <groupId>com.zetcode</groupId>
    <artifactId>springbootrestcontrollerex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>
        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-web</artifactId>
        </dependency>
    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>
</project>

```

这是 Maven `pom.xml`文件。 `spring-boot-starter-parent`是父 POM，它为使用 Maven 构建的应用提供依赖关系和插件管理。 `spring-boot-starter-web`是使用 Spring MVC 构建 Web（包括 RESTful）应用的入门工具。 它使用 Tomcat 作为默认的嵌入式容器。 `spring-boot-maven-plugin`将 Spring 应用打包到可执行的 JAR 或 WAR 归档文件中。

`com/zetcode/model/City.java`

```java
package com.zetcode.model;

import java.util.Objects;

public class City {

    private Long id;
    private String name;
    private int population;

    public City() {
    }

    public City(Long id, String name, int population) {
        this.id = id;
        this.name = name;
        this.population = population;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getPopulation() {
        return population;
    }

    public void setPopulation(int population) {
        this.population = population;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        City city = (City) o;
        return population == city.population &&
                Objects.equals(id, city.id) &&
                Objects.equals(name, city.name);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, name, population);
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("City{");
        sb.append("id=").append(id);
        sb.append(", name='").append(name).append('\'');
        sb.append(", population=").append(population);
        sb.append('}');
        return sb.toString();
    }
}

```

这是一个`City` bean。 它具有`id`，`name`和`population`属性。

`com/zetcode/controller/MyController.java`

```java
package com.zetcode.controller;

import com.zetcode.model.City;
import com.zetcode.service.ICityService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

@RestController
public class MyController {

    @Autowired
    private ICityService cityService;

    @GetMapping(value = "/cities")
    public List<City> getCities() {

        List<City> cities = cityService.findAll();

        return cities;
    }
}

```

这是`MyController`。 它以 JSON 格式返回城市列表。

```java
@RestController
public class MyController {

```

`MyController`带有`@RestController`注解。

```java
@Autowired
private ICityService cityService;

```

我们用`@Autowired`注解注入`CityService`。

```java
@GetMapping(value = "/cities")
public List<City> getCities() {

```

`getCities()`方法映射到`getCities` URL 模式； 它返回城市列表，并通过消息转换器将其转换为 JSON。

`com/zetcode/service/ICityService.java`

```java
package com.zetcode.service;

import com.zetcode.bean.City;
import java.util.List;

public interface ICityService {

    public List<City> findAll();
}

```

`ICityService`包含`findAll()`合约方法。

`com/zetcode/service/CityService.java`

```java
package com.zetcode.service;

import com.zetcode.model.City;
import java.util.ArrayList;
import java.util.List;
import org.springframework.stereotype.Service;

@Service
public class CityService implements ICityService {

    @Override
    public List<City> findAll() {

        var cities = new ArrayList<City>();

        cities.add(new City(1L, "Bratislava", 432000));
        cities.add(new City(2L, "Budapest", 1759000));
        cities.add(new City(3L, "Prague", 1280000));
        cities.add(new City(4L, "Warsaw", 1748000));
        cities.add(new City(5L, "Los Angeles", 3971000));
        cities.add(new City(6L, "New York", 8550000));
        cities.add(new City(7L, "Edinburgh", 464000));
        cities.add(new City(8L, "Berlin", 3671000));

        return cities;
    }
}

```

`CityService`包含`findAll()`方法的实现。

`resources/static/index.html`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Home page</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
    </head>
    <body>
        <p>
            <a href="cities">Get all cities</a>
        </p>
    </body>
</html>

```

这是主页。 它包含一个获取所有城市的链接。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application  {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`是设置 Spring Boot 应用的入口。 `@SpringBootApplication`注解启用自动配置和组件扫描。 在扫描过程中，将查找`@RestController`注解，并从`MyController`类创建一个 Spring bean。

```java
$ mvn spring-boot:run 

```

应用运行后，我们可以导航到`localhost:8080`。

在本教程中，我们展示了如何在 Spring 应用中使用`@RestController`注解。 您可能也对相关教程感兴趣： [Spring Boot `RestTemplate`教程](/springboot/resttemplate/)， [Spring Boot `@Controller`教程](/springboot/controller/)， [Spring Boot `@ExceptionHandler`教程](/springboot/exceptionhandler/)， [Java 教程](/lang/java/) 或列出[所有 Spring Boot 教程](/all/#springboot)。