# Spring Boot MySQL 教程

> 原文： [http://zetcode.com/springboot/mysql/](http://zetcode.com/springboot/mysql/)

Spring Boot MySQL 教程展示了如何在 Spring Boot 应用中使用 MySQL 数据库。

Spring 是用于创建企业应用的流行 Java 应用框架。 Spring Boot 是 Spring 框架的演进，可帮助您轻松创建独立的，生产级的基于 Spring 的应用。

## MySQL

MySQL 是领先的开源数据库管理系统。 它是一个多用户，多线程的数据库管理系统。 MySQL 在网络上特别流行。 它是非常流行的 LAMP 平台的一部分，该平台包括 Linux，Apache，MySQL 和 PHP。 MySQL 数据库在最重要的 OS 平台上可用。 它可以在 BSD Unix，Linux，Windows 和 Mac 上运行。

## MySQL 设置

我们将展示如何在 Debian Linux 系统上安装 MySQL 数据库。

```java
$ sudo apt-get install mysql-server

```

此命令将安装 MySQL 服务器和相关包。

```java
$ sudo service mysql start
$ sudo service mysql stop

```

这两个命令用于启动和停止 MySQL。

```java
$ sudo service mysql status

```

我们使用`service mysql status`命令检查数据库的状态。

```java
$ mysql -u root -p

```

现在我们需要重置根密码。 我们启动 mysql 命令行工具。 （服务器必须正在运行。）我们以`root`用户身份连接。

```java
mysql> SET PASSWORD = PASSWORD('newpassowrd');

```

我们为`root`设置了新密码。

```java
$ mysql_secure_installation

```

我们可以使用`mysql_secure_installation`来提高 MySQL 服务器的安全性。 我们可以选择改进 MySQL 的`root`密码，删除匿名用户帐户，禁用 localhost 外部的`root`登录以及删除测试数据库。

```java
mysql> CREATE DATABASE testdb;

```

我们创建一个新的`testdb`数据库。

```java
mysql> CREATE USER user12@localhost IDENTIFIED BY 's$cret';
mysql> GRANT ALL ON testdb.* TO user12@localhost;

```

我们创建一个新的 MySQL 用户并将其特权授予`testdb`数据库。

## 创建 MySQL 表

现在，我们将创建一个名为`cities`的新 MySQL 表。

`cities_mysql.sql`

```java
DROP TABLE IF EXISTS cities;
CREATE TABLE cities(id INT PRIMARY KEY AUTO_INCREMENT, name VARCHAR(255),
    population INT);

INSERT INTO cities(name, population) VALUES('Bratislava', 432000);
INSERT INTO cities(name, population) VALUES('Budapest', 1759000);
INSERT INTO cities(name, population) VALUES('Prague', 1280000);
INSERT INTO cities(name, population) VALUES('Warsaw', 1748000);
INSERT INTO cities(name, population) VALUES('Los Angeles', 3971000);
INSERT INTO cities(name, population) VALUES('New York', 8550000);
INSERT INTO cities(name, population) VALUES('Edinburgh', 464000);
INSERT INTO cities(name, population) VALUES('Berlin', 3671000);

```

这是用于创建`cities`表的 SQL。

```java
mysql> use testdb;
mysql> source cities_mysql.sql

```

使用`source`命令，执行 SQL 语句。

## Spring Boot MySQL 示例

以下应用是一个简单的 Spring Boot Web 应用，它使用 MySQL 数据库。 我们有一个主页，带有一个链接，用于显示数据库表中的数据。 我们使用 Freemarker 模板系统将数据与 HTML 连接。

```java
pom.xml
src
└── main
    ├── java
    │   └── com
    │       └── zetcode
    │           ├── Application.java
    │           ├── model
    │           │   └── City.java
    │           ├── controller
    │           │   └── MyController.java
    │           ├── repository
    │           │   └── CityRepository.java
    │           └── service
    │               ├── CityService.java
    │               └── ICityService.java
    └── resources
        ├── application.properties
        ├── static
        │   └── index.html
        └── templates
            └── showCities.ftl

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
http://maven.apache.org/xsd/maven-4.0.0.xsd">

    <modelVersion>4.0.0</modelVersion>
    <groupId>com.zetcode</groupId>
    <artifactId>springbootmysqlex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>
        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-web</artifactId>
        </dependency>

        <dependency>
            <groupId>mysql</groupId>
            <artifactId>mysql-connector-java</artifactId>
            <scope>runtime</scope>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-freemarker</artifactId>
        </dependency>

        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter-data-jpa</artifactId>
        </dependency>
    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>
</project>

```

Spring Boot 启动器是一组方便的依赖项描述符，可以极大地简化 Maven 配置。 `spring-boot-starter-parent`具有 Spring Boot 应用的一些常用配置。 `spring-boot-starter-web`是使用 Spring MVC 构建 Web（包括 RESTful）应用的入门工具。 `spring-boot-starter-freemarker`是使用 Freemarker 视图构建 MVC Web 应用的入门工具。 `spring-boot-starter-data-jpa`是将 Spring Data JPA 与 Hibernate 结合使用的入门工具。

`mysql-connector-java`依赖性是针对 MySQL 数据库驱动程序的。

`spring-boot-maven-plugin`在 Maven 中提供了 Spring Boot 支持，使我们可以打包可执行的 JAR 或 WAR 档案。 它的`spring-boot:run`目标运行 Spring Boot 应用。

`resources/application.properties`

```java
spring.main.banner-mode=off
logging.level.org.springframework=ERROR

spring.datasource.url=jdbc:mysql://localhost:3306/testdb?useSSL=false
spring.datasource.username=user12
spring.datasource.password=s$cret

```

在`application.properties`文件中，我们编写了 Spring Boot 应用的各种配置设置。 使用`spring.main.banner-mode`属性，我们可以关闭 Spring 横幅。 使用`logging.level.org.springframework`，我们将 spring 框架的日志记录级别设置为`ERROR`。 在 spring 数据源属性中，我们设置了 MySQL 数据源。

`com/zetcode/model/City.java`

```java
package com.zetcode.model;

import java.util.Objects;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Table;

@Entity
@Table(name = "cities")
public class City {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;

    private String name;
    private int population;

    public City() {
    }

    public City(Long id, String name, int population) {
        this.id = id;
        this.name = name;
        this.population = population;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getPopulation() {
        return population;
    }

    public void setPopulation(int population) {
        this.population = population;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 79 * hash + Objects.hashCode(this.id);
        hash = 79 * hash + Objects.hashCode(this.name);
        hash = 79 * hash + this.population;
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (this == obj) {
            return true;
        }
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final City other = (City) obj;
        if (this.population != other.population) {
            return false;
        }
        if (!Objects.equals(this.name, other.name)) {
            return false;
        }
        return Objects.equals(this.id, other.id);
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("City{");
        sb.append("id=").append(id);
        sb.append(", name='").append(name).append('\'');
        sb.append(", population=").append(population);
        sb.append('}');
        return sb.toString();
    }
}

```

这是`City`实体。 每个实体必须至少定义两个注解：`@Entity`和`@Id`。

```java
@Entity
@Table(name = "cities")
public class City {

```

`@Entity`注解指定该类是一个实体，并映射到数据库表，而`@Table`注解指定要用于映射的数据库表的名称。

```java
@Id
@GeneratedValue(strategy = GenerationType.AUTO)
private Long id;

```

实体的主键由`@Id`注解指定。 `@GeneratedValue`提供了一种用于生成主键值的策略。

`com/zetcode/CityRepository.java`

```java
package com.zetcode.repository;

import com.zetcode.model.City;
import org.springframework.data.repository.CrudRepository;
import org.springframework.stereotype.Repository;

@Repository
public interface CityRepository extends CrudRepository<City, Long> {

}

```

通过从 Spring `CrudRepository`扩展，我们将为我们的数据存储库实现一些方法，包括`findAll()`。 这样，我们节省了大量样板代码。

`com/zetcode/service/ICityService.java`

```java
package com.zetcode.service;

import com.zetcode.model.City;
import java.util.List;

public interface ICityService {

    List<City> findAll();
}

```

`ICityService`提供`findAll()`契约方法声明，以从数据源获取所有城市。

`com/zetcode/service/CityService.java`

```java
package com.zetcode.service;

import com.zetcode.model.City;
import com.zetcode.repository.CityRepository;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class CityService implements ICityService {

    @Autowired
    private CityRepository repository;

    @Override
    public List<City> findAll() {

        return (List<City>) repository.findAll();
    }
}

```

`CityService`包含`findAll()`方法的实现。 我们使用存储库从数据库检索数据。

```java
@Autowired
private CityRepository repository;

```

注入`CityRepository`。

```java
return (List<City>) repository.findAll();

```

存储库的`findAll()`方法返回城市列表。

`com/zetcode/MyController.java`

```java
package com.zetcode.controller;

import com.zetcode.model.City;
import com.zetcode.service.ICityService;
import java.util.List;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;

@Controller
public class MyController {

    @Autowired
    private ICityService cityService;

    @GetMapping("/showCities")
    public String findCities(Model model) {

        var cities = (List<City>) cityService.findAll();

        model.addAttribute("cities", cities);

        return "showCities";
    }
}

```

`MyController`类用`@Controller`注解。

```java
@Autowired
private ICityService cityService;

```

我们将`ICityService`注入`countryService`字段。

```java
@GetMapping("/showCities")
public String findCities(Model model) {

    var cities = (List<City>) cityService.findAll();

    model.addAttribute("cities", cities);

    return "showCities";
}

```

我们将带有`/showCities`路径的请求映射到控制器的`findCities()`方法。 `@GetMapping`注解将 GET 请求映射到该方法。 该模型将获取城市列表，并将处理过程发送到`showCities.ftl` Freemarker 模板文件。

`resources/templates/showCities.ftl`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Cities</title>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        </head>
    <body>
        <h2>List of cities</h2>

        <table>
            <tr>
                <th>Id</th>
                <th>Name</th>
                <th>Population</th>
            </tr>

            <#list cities as city>
                <tr>
                    <td>${city.id}</td>
                    <td>${city.name}</td>
                    <td>${city.population}</td>
                </tr>
            </#list>
        </table>
    </body>
</html>

```

在`showCities.ftl`模板文件中，我们在 HTML 表中显示数据。

`resources/static/index.html`

```java
<!DOCTYPE html>
<html>
    <head>
        <title>Home page</title>
        <meta charset="UTF-8"/>
        <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
    </head>
    <body>
        <a href="showCities">Show cities</a>
    </body>
</html>

```

`index.html`中有一个链接，显示所有城市。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`设置 Spring Boot 应用。 `@SpringBootApplication`启用自动配置和组件扫描。

```java
$ mvn spring-boot:run

```

应用运行后，我们可以导航到`localhost:8080`。

在本教程中，我们展示了如何在 Spring Boot 应用中使用 MySQL 数据库。 您可能也对相关教程感兴趣：

*   [Spring Boot PostgreSQL 教程](/springboot/postgresql/)
*   [Spring Boot H2 REST 教程](/articles/springbootresth2/)
*   [Spring Boot Thymeleaf 教程](/articles/springbootthymeleaf/)
*   [Spring Web 应用介绍](/articles/springwebfirst/)
*   [Spring Boot RESTFul 应用](/articles/springbootrestsimple/)

查找[所有 Spring Boot 教程](/all/#springboot)。