# Spring Boot `@Order`教程

> 原文： [http://zetcode.com/springboot/order/](http://zetcode.com/springboot/order/)

Spring Boot `@Order`教程展示了如何使用`@Order`注解来规定 bean。

Spring 是流行的 Java 应用框架，而 Spring Boot 是 Spring 的演进，可以帮助轻松地创建独立的，生产级的基于 Spring 的应用。

## `@Order`

`@Order`定义带注解的组件的排序顺序。 `value()`是可选的，表示订单值。 较低的值具有较高的优先级。

## Spring Boot `@Order`示例

以下应用命令执行`CommandLineRunner`的 bean 的执行。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │               Application.java
│   │               MyRunner.java
│   │               MyRunner2.java
│   └───resources
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
http://maven.apache.org/xsd/maven-4.0.0.xsd">

    <modelVersion>4.0.0</modelVersion>
    <groupId>com.zetcode</groupId>
    <artifactId>springbootorder</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>jar</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
    </properties>

    <parent>
        <groupId>org.springframework.boot</groupId>
        <artifactId>spring-boot-starter-parent</artifactId>
        <version>2.1.1.RELEASE</version>
    </parent>

    <dependencies>
        <dependency>
            <groupId>org.springframework.boot</groupId>
            <artifactId>spring-boot-starter</artifactId>
        </dependency>
    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.springframework.boot</groupId>
                <artifactId>spring-boot-maven-plugin</artifactId>
            </plugin>
        </plugins>
    </build>
</project>

```

这是 Maven `pom.xml`文件。

`com/zetcode/MyRunner.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

@Component
@Order(value = 2)
public class MyRunner implements CommandLineRunner {

    private static final Logger logger = LoggerFactory.getLogger(MyRunner.class);

    @Override
    public void run(String... args) throws Exception {

        logger.info("Running MyRunner");
    }
}

```

在应用启动时启动 Bean。 使用`@Order`注解，我们为其赋予了优先级。

`com/zetcode/MyRunner2.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.CommandLineRunner;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;

@Component
@Order(value = 1)
public class MyRunner2 implements CommandLineRunner {

    private static final Logger logger = LoggerFactory.getLogger(MyRunner.class);

    @Override
    public void run(String... args) throws Exception {
        logger.info("Running MyRunner2");
    }
}

```

这是`MyRunner2`。 通过`@Order`设置了更高的优先级，因此在`MyRunner`之前执行。

`com/zetcode/Application.java`

```java
package com.zetcode;

import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class Application  {

    public static void main(String[] args) {
        SpringApplication.run(Application.class, args);
    }
}

```

`Application`是设置 Spring Boot 应用的入口。

我们使用`mvn -q spring-boot:run`运行该应用。

在本教程中，我们展示了如何使用`@Order`接口来设置 bean 的执行顺序。 您可能也对相关教程感兴趣： [Spring Boot `@Repository`教程](/springboot/repository/)， [Spring Boot `CommandLineRunner`教程](/springboot/commandlinerunner/)， [Java 教程](/lang/java/)或列出[所有 Spring Boot 教程](/all/#springboot)。