# Spring 转发教程

> 原文： [http://zetcode.com/spring/forward/](http://zetcode.com/spring/forward/)

Spring 转发教程展示了如何在 Spring Web 应用中转发请求。

Spring 是用于创建企业应用的流行 Java 应用框架。

## 转发

视图名称中的特殊`forward:`前缀将转发到其他 URL。 转发 URL 可在同一服务器内部内部传输请求，而无需涉及客户端浏览器。 转发的执行频率少于重定向。

## 重定向与转发

基本上可以通过三种方式处理请求：a）由 Spring 在控制器动作中解决，b）转发至其他控制器动作，c）重定向至客户端以获取另一个 URL。

向前：

*   由 Spring 内部执行
*   浏览器完全不知道转发，因此其原始 URL 保持不变
*   浏览器重新加载结果页面会重复原始请求，并带有原始 URL
*   请求中发送的数据可用于转发的操作

重定向：

*   是一个两步过程
*   Spring 指示浏览器获取另一个 URL，该 URL 与原始 URL 不同
*   浏览器重新加载第二个 URL 不会重复原始请求，而是获取第二个 URL
*   在原始请求范围内发送的数据不适用于第二个请求

## Spring 转发的例子

提交表单后，以下应用使用转发到其他 URL。 它执行带有`forward:`前缀的转发。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           ├───config
│   │           │       MyWebInitializer.java
│   │           │       WebConfig.java
│   │           └───controller
│   │                   MyController.java
│   └───resources
│       │   logback.xml
│       │
│       └───templates
│               show.ftl
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0 
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springforwardex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>war</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>
    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>javax.servlet</groupId>
            <artifactId>javax.servlet-api</artifactId>
            <version>4.0.1</version>
            <scope>provided</scope>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-webmvc</artifactId>
            <version>5.1.3.RELEASE</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context-support</artifactId>
            <version>5.1.3.RELEASE</version>
        </dependency>

        <dependency>
            <groupId>org.freemarker</groupId>
            <artifactId>freemarker</artifactId>
            <version>2.3.28</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>

            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-war-plugin</artifactId>
                <version>3.2.2</version>
            </plugin>

            <plugin>
                <groupId>org.eclipse.jetty</groupId>
                <artifactId>jetty-maven-plugin</artifactId>
                <version>9.4.14.v20181114</version>
            </plugin>

        </plugins>
    </build>
</project>

```

在`pom.xml`中，我们具有必要的依赖性。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`com/zetcode/config/MyWebInitializer.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.support.AbstractAnnotationConfigDispatcherServletInitializer;

@Configuration
public class MyWebInitializer extends
        AbstractAnnotationConfigDispatcherServletInitializer {

    @Override
    protected Class<?>[] getRootConfigClasses() {
        return null;
    }

    @Override
    protected Class<?>[] getServletConfigClasses() {

        return new Class[]{WebConfig.class};
    }

    @Override
    protected String[] getServletMappings() {

        return new String[]{"/"};
    }
}

```

`MyWebInitializer`注册 Spring `DispatcherServlet`，它是 Spring Web 应用的前端控制器。

```java
@Override
protected Class<?>[] getServletConfigClasses() {

    return new Class[]{WebConfig.class};
}

```

`getServletConfigClasses()`返回 Web 配置类。

`com/zetcode/config/WebConfig.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import org.springframework.web.servlet.view.freemarker.FreeMarkerConfigurer;
import org.springframework.web.servlet.view.freemarker.FreeMarkerViewResolver;

@Configuration
@EnableWebMvc
@ComponentScan(basePackages = {"com.zetcode"})
public class WebConfig implements WebMvcConfigurer {

    @Bean
    public FreeMarkerViewResolver freemarkerViewResolver() {

        var resolver = new FreeMarkerViewResolver();
        resolver.setCache(true);
        resolver.setSuffix(".ftl");
        return resolver;
    }

    @Bean
    public FreeMarkerConfigurer freemarkerConfig() {

        var freeMarkerConfigurer = new FreeMarkerConfigurer();
        freeMarkerConfigurer.setTemplateLoaderPath("classpath:/templates/");
        return freeMarkerConfigurer;
    }
}

```

`WebConfig`配置 Freemarker。 我们将模板文件的位置设置为`classpath`上的`templates`目录。 （`resources`在类路径上。）

`com/zetcode/controller/MyController.java`

```java
package com.zetcode.controller;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

@Controller
public class MyController {

    private static final Logger logger = LoggerFactory.getLogger(MyController.class);

    @GetMapping(value = "/sendname")
    public String send(@RequestParam(defaultValue = "guest") String name, Model model) {

        logger.info("sendname called");

        model.addAttribute("name", name);

        return "forward:/newpage";
    }

    @GetMapping(value = "/newpage")
    public String newpage() {

        logger.info("newpage called");

        return "show";
    }
}

```

`MyController`提供了两个 GET 映射。

```java
@GetMapping(value = "/sendname")
public String send(@RequestParam(defaultValue = "guest") String name, Model model) {

    logger.info("send name called");

    model.addAttribute("name", name);

    return "forward:/newpage";
}

```

的请求发送至`/sendname`由`send()`操作处理。 读取请求参数并将其添加到模型。 最后，它被转发到新的控制器操作。

```java
@GetMapping(value = "/newpage")
public String newpage() {

    logger.info("newpage called");

    return "show";
}

```

`newpage()`操作将请求解析为`show`视图。

`resources/templates/show.ftl`

```java
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <title>Show</title>
    <meta name="viewport" content="width=device-width, initial-scale=1">
</head>
<body>

<p>
User name: ${name}
</p>

</body>
</html>

```

`show.ftl`显示用户名。

```java
$ mvn jetty:run

```

我们运行服务器并找到`localhost:8080/sendname?name=Peter`。

```java
09:52:11.636 INFO  com.zetcode.controller.MyController - sendname called 
09:52:11.653 INFO  com.zetcode.controller.MyController - newpage called 

```

服务器日志包含这些行。

在本教程中，我们在 Spring 控制器中执行了正向操作。

您可能也对这些相关教程感兴趣： [Spring 重定向教程](/spring/redirect/)， [Java 教程](/lang/java/)或列出[所有 Spring 教程](/all/#spring)。