# Spring `List` XML 注入教程

> 原文： [http://zetcode.com/spring/injectlistxml/](http://zetcode.com/spring/injectlistxml/)

Spring 注入列表 XML 教程展示了如何在 XML 配置中将列表注入到应用上下文中。

Spring 是用于创建企业应用的流行 Java 应用框架。

## util 模式

util XML 模式处理常见的实用程序配置问题，例如配置集合或引用常量。

## Spring 注入列表示例

该应用从 Spring XML 配置注入一个 List 集合。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>injectlistex</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`和`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`src/main/resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} [%thread] %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`src/main/resources/my-beans.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<beans xmlns="http://www.springframework.org/schema/beans"
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        xmlns:context="http://www.springframework.org/schema/context"
        xmlns:util="http://www.springframework.org/schema/util"
        xsi:schemaLocation="http://www.springframework.org/schema/beans
            http://www.springframework.org/schema/beans/spring-beans.xsd
            http://www.springframework.org/schema/context
            http://www.springframework.org/schema/context/spring-context.xsd
            http://www.springframework.org/schema/util
            http://www.springframework.org/schema/util/spring-util.xsd">

    <context:component-scan base-package="com.zetcode"/>

    <util:list id="myColours" value-type="java.lang.String">
        <value>red</value>
        <value>green</value>
        <value>blue</value>
        <value>yellow</value>
        <value>brown</value>
        <value>orange</value>
    </util:list>
</beans>

```

在`my-beans.xml`配置文件中，我们定义了颜色列表。 列表类型为`java.util.String`，并使用`value`标签指定值。 我们需要导入适当的架构定义。

`src/main/java/com/zetcode/Application.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.context.support.GenericXmlApplicationContext;

import java.util.List;

public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {

        var ctx = new GenericXmlApplicationContext("my-beans.xml");
        var colours = (List<String>) ctx.getBean("myColours");

        logger.info(colours.toString());

        ctx.close();
    }
}

```

在`Application`类中，我们使用`getBean()`检索 bean 列表并将其记录到控制台。

```java
$ mvn -q exec:java
21:28:19.770 [com.zetcode.Application.main()] INFO  com.zetcode.Application - [red, green, blue, yellow, brown, orange]    

```

我们运行该应用。

在本教程中，我们展示了如何使用 XML 配置将值列表注入到应用中。

您可能对以下相关教程感兴趣：[经典 Spring 应用中的 JdbcTemplate](/articles/springjdbctemplate/) ， [Spring 单例范围 bean](/spring/singletonscope/) ， [Spring BeanDefinitionBuilder 教程](/spring/beandefinitionbuilder/)， [Spring HikariCP 教程](/articles/springhikaricp/) 和 [Java 教程](/lang/java/)。