# Spring `BeanDefinitionBuilder`教程

> 原文： [http://zetcode.com/spring/beandefinitionbuilder/](http://zetcode.com/spring/beandefinitionbuilder/)

Spring `BeanDefinitionBuilder`教程展示了如何使用`BeanDefinitionBuilder`以编程方式创建新的 Spring Bean。

Spring 是用于创建企业应用的流行 Java 应用框架。

## `BeanDefinitionBuilder`

`BeanDefinitionBuilder`用于以编程方式创建新的 Spring bean。 它利用了构建器模式。

## Spring `BeanDefinitionBuilder`示例

该应用创建一个简单的字符串 bean。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
            xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
            xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
            http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>beanbuilderex</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>

    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

在`pom.xml`文件中，我们具有基本的 Spring 依赖项`spring-core`和`spring-context`和日志记录`logback-classic`依赖项。

`exec-maven-plugin`用于在命令行上从 Maven 执行 Spring 应用。

`src/main/resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} [%thread] %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`src/main/java/com/zetcode/Application.java`

```java
package com.zetcode;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.ComponentScan;

@ComponentScan(basePackages = "com.zetcode")
public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {

        var ctx = new AnnotationConfigApplicationContext(Application.class);

        var beanFactory = (BeanDefinitionRegistry) ctx.getBeanFactory();

        beanFactory.registerBeanDefinition("myBean",
                BeanDefinitionBuilder.genericBeanDefinition(String.class)
                        .addConstructorArgValue("This is my bean")
                        .setScope("prototype")
                        .getBeanDefinition()
        );

        logger.info("{}", ctx.getBean("myBean"));

        ctx.close();
    }
}

```

该示例使用`BeanDefinitionBuilder`创建一个类型为`String`的简单 bean。

```java
var beanFactory = (BeanDefinitionRegistry) ctx.getBeanFactory();

```

使用`getBeanFactory()`从应用上下文中检索 bean 工厂。

```java
beanFactory.registerBeanDefinition("myBean",
    BeanDefinitionBuilder.genericBeanDefinition(String.class)
            .addConstructorArgValue("This is my bean")
            .setScope("prototype")
            .getBeanDefinition()
);

```

我们向`registerBeanDefinition()`注册了一个新 bean。

```java
logger.info("{}", ctx.getBean("myBean"));

```

我们使用`getBean()`从应用上下文中获取 bean 并将其记录下来。

```java
$ mvn -q exec:java
20:51:05.970 [com.zetcode.Application.main()] INFO com.zetcode.Application - This is my bean    

```

我们运行该应用。

在本教程中，我们使用`BeanDefinitionBuilder`以编程方式注册了一个新的 Spring bean。

您可能也对这些相关教程感兴趣： [Spring 注入列表 XML 教程](/spring/injectlistxml/)， [Spring `BeanFactory`教程](/spring/beanfactory/)，[经典 Spring 应用中的`JdbcTemplate`](/articles/springjdbctemplate/) ， [Spring 单例范围 bean](/spring/singletonscope/) ， [Spring HikariCP 教程](/articles/springhikaricp/)和 [Java 教程](/lang/java/)。