# Spring `EmbeddedDatabaseBuilder`教程

> 原文： [http://zetcode.com/spring/embeddeddatabasebuilder/](http://zetcode.com/spring/embeddeddatabasebuilder/)

Spring `EmbeddedDatabaseBuilder`教程展示了如何使用`EmbeddedDatabaseBuilder`在 Spring 应用中创建 H2 嵌入式数据库。

Spring 是流行的 Java 应用框架。

## `EmbeddedDatabaseBuilder`

`EmbeddedDatabaseBuilder`是一个 Spring 构建器，它提供方便的 API 在 Spring 应用中创建嵌入式数据库。

## Spring `EmbeddedDatabaseBuilder`示例

以下示例使用`EmbeddedDatabaseBuilder`构建嵌入式 H2 数据库。 我们使用 Spring `JdbcTemplate`与数据库进行交互。

```java
pom.xml
src    
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           │   Application.java
│   │           ├───config
│   │           │       DBConfig.java
│   │           └───model
│   │                   Car.java
│   └───resources
│       │   logback.xml
│       └───db
│               create-db.sql
│               insert-data.sql
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>embeddeddatabasebuilderex</artifactId>
    <version>1.0-SNAPSHOT</version>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>
    </properties>

    <dependencies>

        <dependency>
            <groupId>com.h2database</groupId>
            <artifactId>h2</artifactId>
            <version>1.4.197</version>
        </dependency>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-context</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-core</artifactId>
            <version>${spring-version}</version>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-jdbc</artifactId>
            <version>${spring-version}</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>
            <plugin>
                <groupId>org.codehaus.mojo</groupId>
                <artifactId>exec-maven-plugin</artifactId>
                <version>1.6.0</version>
                <configuration>
                    <mainClass>com.zetcode.Application</mainClass>
                </configuration>
            </plugin>
        </plugins>
    </build>

</project>

```

这是我们 Spring 应用的 Maven 构建文件。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

这是 Logback 配置文件。

`EmbeddedDatabaseBuilder`将使用以下两个 SQL 脚本来创建`cars`表并将数据插入其中。

`resources/db/schema.sql`

```java
CREATE TABLE cars(id INT PRIMARY KEY AUTO_INCREMENT, name VARCHAR(150), price INT);

```

`schema.sql`创建数据库表。

`resources/db/data.sql`

```java
INSERT INTO cars(name, price) VALUES('Audi', 52642);
INSERT INTO cars(name, price) VALUES('Mercedes', 57127);
INSERT INTO cars(name, price) VALUES('Skoda', 9000);
INSERT INTO cars(name, price) VALUES('Volvo', 29000);
INSERT INTO cars(name, price) VALUES('Bentley', 350000);
INSERT INTO cars(name, price) VALUES('Citroen', 21000);
INSERT INTO cars(name, price) VALUES('Hummer', 41400);
INSERT INTO cars(name, price) VALUES('Volkswagen', 21600);

```

`data.sql`将数据插入表中。

`com/zetcode/model/Car.java`

```java
package com.zetcode.model;

import java.util.Objects;

public class Car {

    private Long id;
    private String name;
    private int price;

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public int getPrice() {
        return price;
    }

    public void setPrice(int price) {
        this.price = price;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Car car = (Car) o;
        return price == car.price &&
                Objects.equals(id, car.id) &&
                Objects.equals(name, car.name);
    }

    @Override
    public int hashCode() {
        return Objects.hash(id, name, price);
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("Car{");
        sb.append("id=").append(id);
        sb.append(", name='").append(name).append('\'');
        sb.append(", price=").append(price);
        sb.append('}');
        return sb.toString();
    }
}

```

这是一个`Car`类。

`com/zetcode/config/DBConfig.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseBuilder;
import org.springframework.jdbc.datasource.embedded.EmbeddedDatabaseType;

import javax.sql.DataSource;

@Configuration
public class DBConfig  {

    @Bean
    public DataSource dataSource() {

        var builder = new EmbeddedDatabaseBuilder();
        var db = builder
                .setType(EmbeddedDatabaseType.H2) // HSQL or DERBY
                .addScript("db/schema.sql")
                .addScript("db/data.sql")
                .build();
        return db;
    }

    @Bean
    public JdbcTemplate createJdbcTeamplate() {

        var template = new JdbcTemplate();
        template.setDataSource(dataSource());

        return template;
    }
}

```

`DBConfig`使用`EmbeddedDatabaseBuilder`创建嵌入式 H2 数据库。 我们还创建了一个`JdbcTemplate` bean。

```java
@Bean
public DataSource dataSource() {

    var builder = new EmbeddedDatabaseBuilder();
    var db = builder
            .setType(EmbeddedDatabaseType.H2) // HSQL or DERBY
            .addScript("db/schema.sql")
            .addScript("db/data.sql")
            .build();
    return db;
}

```

该方法使用`EmbeddedDatabaseBuilder`的`build()`方法创建数据源。 我们使用`setType()`指定数据库类型，并使用`addScript()`方法添加 SQL 脚本。

```java
@Bean
public JdbcTemplate createJdbcTeamplate() {

    var template = new JdbcTemplate();
    template.setDataSource(dataSource());

    return template;
}

```

此方法生成一个新的`JdbcTemplate`。 我们使用`setDataSource()`将生成的数据源设置为模板。

`com/zetcode/Application.java`

```java
package com.zetcode;

import com.zetcode.model.Car;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.AnnotationConfigApplicationContext;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;

@ComponentScan(basePackages = "com.zetcode")
public class Application {

    private static final Logger logger = LoggerFactory.getLogger(Application.class);

    public static void main(String[] args) {

        var ctx = new AnnotationConfigApplicationContext(Application.class);
        var app = ctx.getBean(Application.class);

        app.run();

        ctx.close();
    }

    @Autowired
    private JdbcTemplate jdbcTemplate;

    private void run() {

        var sql = "SELECT * FROM cars";

        var cars = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(Car.class));

        cars.forEach(car -> logger.info("{}", car));
    }
}

```

在应用中，我们执行查询以查找所有汽车。

```java
@Autowired
private JdbcTemplate jdbcTemplate;

```

注入了`JdbcTemplate` bean。

```java
var cars = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(Car.class));

```

使用`JdbcTemplate`的`query()`方法执行 SQL 查询。

```java
cars.forEach(car -> logger.info("{}", car));

```

检索到的汽车将写入控制台。

```java
$ mvn -q exec:java
19:13:35.753 INFO  com.zetcode.Application - Car{id=1, name='Audi', price=52642}
19:13:35.768 INFO  com.zetcode.Application - Car{id=2, name='Mercedes', price=57127}
19:13:35.768 INFO  com.zetcode.Application - Car{id=3, name='Skoda', price=9000}
19:13:35.768 INFO  com.zetcode.Application - Car{id=4, name='Volvo', price=29000}
19:13:35.768 INFO  com.zetcode.Application - Car{id=5, name='Bentley', price=350000}
19:13:35.768 INFO  com.zetcode.Application - Car{id=6, name='Citroen', price=21000}
19:13:35.768 INFO  com.zetcode.Application - Car{id=7, name='Hummer', price=41400}
19:13:35.768 INFO  com.zetcode.Application - Car{id=8, name='Volkswagen', price=21600}

```

我们运行该应用。

在本教程中，我们使用`EmbeddedDatabaseBuilder`在 Spring 应用中创建嵌入式 H2 数据库。

您可能也对相关教程感兴趣： [`JdbcTemplate`教程](/db/jdbctemplate/)， [Spring `@ComponentScan`教程](/spring/componentscan/)和 [Java 教程](/lang/java/)，