# Spring `@RequestBody`教程

> 原文： [http://zetcode.com/spring/requestbody/](http://zetcode.com/spring/requestbody/)

Spring `@RequestBody`教程展示了如何使用`@RequestBody`注解将方法参数绑定到请求主体。

Spring 是用于创建企业应用的流行 Java 应用框架。

## Spring `@RequestBody`

`@RequestBody`注释将请求主体绑定到方法参数。 序列化/反序列化的过程由`HttpMessageConverter`执行。 另外，可以通过使用`@Valid`注释自变量来应用自动验证。

## Spring `@RequestBody`示例

该应用将表单 POST 和 JSON post 请求的请求主体参数绑定到映射的方法参数。

```java
pom.xml
src
├───main
│   ├───java
│   │   └───com
│   │       └───zetcode
│   │           ├───bean
│   │           │       User.java
│   │           ├───config
│   │           │       MyWebInitializer.java
│   │           │       WebConfig.java
│   │           └───controller
│   │                   MyController.java
│   └───resources
│           logback.xml
└───test
    └───java

```

这是项目结构。

`pom.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<project xmlns="http://maven.apache.org/POM/4.0.0"
         xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
         xsi:schemaLocation="http://maven.apache.org/POM/4.0.0
         http://maven.apache.org/xsd/maven-4.0.0.xsd">
    <modelVersion>4.0.0</modelVersion>

    <groupId>com.zetcode</groupId>
    <artifactId>springrequestbodyex</artifactId>
    <version>1.0-SNAPSHOT</version>
    <packaging>war</packaging>

    <properties>
        <project.build.sourceEncoding>UTF-8</project.build.sourceEncoding>
        <maven.compiler.source>11</maven.compiler.source>
        <maven.compiler.target>11</maven.compiler.target>
        <spring-version>5.1.3.RELEASE</spring-version>
    </properties>

    <dependencies>

        <dependency>
            <groupId>ch.qos.logback</groupId>
            <artifactId>logback-classic</artifactId>
            <version>1.2.3</version>
        </dependency>

        <dependency>
            <groupId>javax.servlet</groupId>
            <artifactId>javax.servlet-api</artifactId>
            <version>4.0.1</version>
            <scope>provided</scope>
        </dependency>

        <dependency>
            <groupId>org.springframework</groupId>
            <artifactId>spring-webmvc</artifactId>
            <version>5.1.3.RELEASE</version>
        </dependency>

        <dependency>
            <groupId>com.fasterxml.jackson.core</groupId>
            <artifactId>jackson-databind</artifactId>
            <version>2.9.8</version>
        </dependency>

    </dependencies>

    <build>
        <plugins>

            <plugin>
                <groupId>org.apache.maven.plugins</groupId>
                <artifactId>maven-war-plugin</artifactId>
                <version>3.2.2</version>
            </plugin>

            <plugin>
                <groupId>org.eclipse.jetty</groupId>
                <artifactId>jetty-maven-plugin</artifactId>
                <version>9.4.14.v20181114</version>
            </plugin>

        </plugins>
    </build>
</project>

```

我们声明必要的依赖关系。 在`HttpMessageConverter`中进行序列化需要`jackson-databind`。 该应用在嵌入式 Jetty 上运行； 因此，我们声明`jetty-maven-plugin`。

`resources/logback.xml`

```java
<?xml version="1.0" encoding="UTF-8"?>
<configuration>
    <logger name="org.springframework" level="ERROR"/>
    <logger name="com.zetcode" level="INFO"/>

    <appender name="consoleAppender" class="ch.qos.logback.core.ConsoleAppender">
        <encoder>
            <Pattern>%d{HH:mm:ss.SSS} %blue(%-5level) %magenta(%logger{36}) - %msg %n
            </Pattern>
        </encoder>
    </appender>

    <root>
        <level value="INFO" />
        <appender-ref ref="consoleAppender" />
    </root>
</configuration>

```

`logback.xml`是 Logback 日志库的配置文件。

`com/zetcode/bean/User.java`

```java
package com.zetcode.bean;

public class User {

    private String name;
    private String occupation;

    public User() {
    }

    public User(String name, String occupation) {
        this.name = name;
        this.occupation = occupation;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getOccupation() {
        return occupation;
    }

    public void setOccupation(String occupation) {
        this.occupation = occupation;
    }

    @Override
    public String toString() {
        final StringBuilder sb = new StringBuilder("User{");
        sb.append("name='").append(name).append('\'');
        sb.append(", occupation='").append(occupation).append('\'');
        sb.append('}');
        return sb.toString();
    }
}

```

在示例中，我们具有`User` bean，它具有`name`和`occupation`属性。

`com/zetcode/config/MyWebInitializer.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.support.AbstractAnnotationConfigDispatcherServletInitializer;

@Configuration
public class MyWebInitializer extends
        AbstractAnnotationConfigDispatcherServletInitializer {

    @Override
    protected Class<?>[] getRootConfigClasses() {
        return null;
    }

    @Override
    protected Class<?>[] getServletConfigClasses() {

        return new Class[]{WebConfig.class};
    }

    @Override
    protected String[] getServletMappings() {

        return new String[]{"/"};
    }
}

```

`MyWebInitializer`注册 Spring `DispatcherServlet`，它是 Spring Web 应用的前端控制器。

`com/zetcode/config/WebConfig.java`

```java
package com.zetcode.config;

import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.web.servlet.config.annotation.EnableWebMvc;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;

@Configuration
@EnableWebMvc
@ComponentScan(basePackages = {"com.zetcode"})
public class WebConfig implements WebMvcConfigurer {

}

```

`WebConfig`通过`@EnableWebMvc`启用 Spring MVC 注解，并为`com.zetcode`包配置组件扫描。

`com/zetcode/controller/MyController.java`

```java
package com.zetcode.controller;

import com.zetcode.bean.User;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class MyController {

    private static final Logger logger = LoggerFactory.getLogger(MyController.class);

    @ResponseStatus(value = HttpStatus.OK)
    @PostMapping(value="/vals")
    public void process(@RequestBody MultiValueMap<String, String> values) {

        logger.info("Values:{}", values);
    }

    @ResponseStatus(value = HttpStatus.OK)
    @PostMapping(value="/user", consumes = MediaType.APPLICATION_JSON_VALUE)
    public void process2(@RequestBody User user) {

        logger.info("User: {}", user);
    }
}

```

在`MyContoller`中，我们有两个 POST 映射。 我们使用`@RequestBody`将请求参数绑定到`MultiValueMap`和`User` bean。 绑定值显示在日志中。

```java
$ mvn jetty:run

```

我们启动服务器。

```java
$ curl -i -d "par1=val1&par2=val2" -X POST http://localhost:8080/vals

```

使用`curl`工具，我们可以创建第一个映射的请求。 这将创建一个表单 POST 数据请求（内容类型为`application/x-www-form-urlencoded`）。

```java
11:57:39.049 INFO  com.zetcode.controller.MyController - Values:{par1=[val1], par2=[val2]}

```

我们得到这个日志。

```java
$ curl -i -H "Content-Type: application/json"  -d "{\"name\":\"John Doe\",\"occupation\":\"gardener\"}" -X POST "http://localhost:8080/user"

```

我们调用第二个映射。 在这里，我们使用 JSON 数据创建一个请求。 请注意，在 Windows 上，我们需要转义双引号。

```java
12:02:33.817 INFO  com.zetcode.controller.MyController - User: User{name='John Doe', occupation='gardener'}

```

这是日志中的输出。

在本教程中，我们使用`@RequestBody`注解将请求属性绑定到方法参数。

您可能也对这些相关教程感兴趣： [Spring `@PostMapping`教程](/spring/postmapping/)， [Java 教程](/lang/java/)或列出[所有 Spring 教程](/all/#spring)。